package lbl.akenti.certs.elements;

import java.io.IOException;
import java.text.ParseException;
import lbl.akenti.util.ParsingException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
  * An <code>AkentiIssuer</code> holds an <code>AkentiPrincipal</code>
  * and the directories where its chain of x509 certs is stored.
  *
  * <pre>
  * AkentiIssuer {
  *   AkentiPrincipal issuer;
  *   vectorlt&Directory&gt  idDirs;
  * }
  *
  * Format:
  *   issuer idDirCount idDirs*
  *
  * </pre>
  *
  * @see AkentiObject
  * @see AkentiPrincipal
  * @see Directory
  *
  * @version 1.1 02/05/01
  *
  * @author Srilekha Mudumbai
  * @author Abdelilah Essiari
  **/
public final class AkentiIssuer extends AkentiObject {

  private AkentiPrincipal issuer;

  private Directory[] identityDirs;

  /**
    * Constructs this principal using default values;
    */
  public AkentiIssuer() {
    this(new AkentiPrincipal());
  }

  /**
    * Constructs this issuer.
    */
  public AkentiIssuer(AkentiPrincipal issuer) {
    this(issuer, null);
  }

  /**
    * Constructs this issuer.
    */
  public AkentiIssuer(AkentiPrincipal issuer, Directory[] identityDirs) {
    if (issuer == null) { 
      throw new IllegalArgumentException();
    }

    this.issuer = issuer;
    this.identityDirs = identityDirs == null? new Directory[0]: identityDirs;
  }

  /**
    * Constructs this issuer.
    *
    * @param source stream used to initialize this object.
    */
  public
  AkentiIssuer(AkentiInputStream source) throws IOException, ParseException {
    readObject(source);
  }

  /**
    * Constructs this issuer.
    *
    * @param element XML element used to parse this object.
    */
  public
  AkentiIssuer(Element element) throws ParseException {
    fromXML(element);
  }

  public DistinguishedName getName() {
    return issuer.getName();
  }

  public DistinguishedName getGuarantor() {
    return issuer.getGuarantor();
  }

  public AkentiPrincipal getIssuer() {
    return issuer;
  }

  public Directory[] getIdentityDirs() {
    return identityDirs;
  }

  /**
    * Tests for equality. Useful in containers such as vectors or sets.
    *
    * Note only the issuer components are compared
    */
  public boolean equals(Object obj) {
    if (obj instanceof AkentiIssuer) {
      AkentiIssuer temp  = (AkentiIssuer) obj;
     
      return issuer.equals(temp.issuer);
    }

    return  false;
  }

  /**
    * Writes this issuer to an <code>AkentiOutputStream</code>.
    *
    * @see AkentiOutputStream
    **/
  public void 
  writeObject(AkentiOutputStream drain) throws IOException {
    issuer.writeObject(drain);
    drain.write(' ');
    drain.write(identityDirs.length); 
    
    for (int i=0; i < identityDirs.length; i++) {
      drain.write(' ');
      identityDirs[i].writeObject(drain);
    }
  }

  /**
    * Initializes this issuer from a <code>AkentiInputStream</code>.
    *
    * @see AkentiInputStream
    */
  private void 
  readObject(AkentiInputStream source) throws IOException, ParseException {
    issuer = new AkentiPrincipal(source);
    
    int len = source.readInt();

    if (len < 0) {
      throw new ParsingException();
    }

    identityDirs = new Directory[len];
    
    for (int i=0; i < identityDirs.length; i++) {
       identityDirs[i] = new Directory(source);
    } 
  }

  /**
    * Populates the XML element <code>akIssue</code>.
    */
  public void toXML(Element akIssuer) {
    issuer.toXML(akIssuer);

    for (int i = 0; i < identityDirs.length; ++i) {
      Document doc = akIssuer.getOwnerDocument();
      Element temp = doc.createElement("URL");

      identityDirs[i].toXML(temp);
      akIssuer.appendChild(temp);
    }
  }

  /**
    * Initializes this issuer from the XML element <code>akIssuer</code>.
    */
  private void fromXML(Element akIssuer) throws ParseException {
    issuer = new AkentiPrincipal(akIssuer);

    NodeList dirs = akIssuer.getElementsByTagName("URL"); 
    int len = dirs.getLength();

    identityDirs = new Directory[len];
      
    for (int i=0; i < len; i++) {
      identityDirs[i] = new Directory((Element) dirs.item(i));
    }
  }

  /**
    * Returns debugging info.
    */
  public String paramString() {
    StringBuffer sbuf = new StringBuffer();

    sbuf.append("[");
    sbuf.append(issuer.paramString());
    sbuf.append(",");
    sbuf.append("[");

    if (identityDirs.length > 0) {
      sbuf.append(identityDirs[0].paramString());
    }

    for (int i=1; i < identityDirs.length; i++) {
      sbuf.append(",");
      sbuf.append(identityDirs[i].paramString());
    }

    sbuf.append("]");
    sbuf.append("]");
    return sbuf.toString();
  }
}
