package lbl.akenti.certs.elements;

import java.io.IOException;
import java.security.Principal;
import java.text.ParseException;
import lbl.akenti.util.ParsingException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

/**
  * An <code>AkentiPrincipal</code> is used to represent an entity
  * in the <code>Akenti</code> world. It consists of two components
  * the name of the entity and the name of the guarantor of
  * that entity. 
  *
  *                                                   <BR>
  * Note:                                             <BR>
  *
  * The names are <code>x509</code> distingushed names.     <BR>
  * The guarantor is the immediate ca of the entity.  <BR>
  *
  * <pre>
  *
  * AkentiPrincipal {
  *   DistinguishedName name
  *   DistinguishedNmae guarantor
  * }
  *
  * Canonical Format:
  *   name guarantor
  *
  * Example: (/C=US/ST=CA/CN=John Doe, /C=US/ST=CA/CN=CA)
  *
  *   will be writen as: /C=US/ST=CA/CN=John\ Doe /C=US/ST=CA/CN=CA
  *
  * </pre>
  *
  * @see AkentiObject
  * @see DistinguishedName
  *
  * @version 1.1 02/05/01
  * @author Srilekha Mudumbai
  * @author Abdelilah Essiari
  **/

final public 
class AkentiPrincipal extends AkentiObject {

  private DistinguishedName name;      // distinguished name of the entity

  private DistinguishedName guarantor; // distinguished name of the guarantor

  /**
    * Constructs this principal using default distingushed names.
    */
  public AkentiPrincipal() {
    this(new DistinguishedName(), new DistinguishedName());
  }

  /**
    * Constructs this principal.
    *
    * @param name principal's distinguished name in X500 format
    * @param guarantor ca's distinguished name in X500 format 
    */
  public AkentiPrincipal(String name, String guarantor) throws ParsingException {
    this(new DistinguishedName(name), new DistinguishedName(guarantor));
  }

  /**
    * Constructs this principal.
    *
    * @param name principal's distinguished name
    * @param guarantor guarantor's distinguished name 
    */
  public AkentiPrincipal(Principal name, Principal guarantor) throws ParsingException {
    this(new DistinguishedName(name), new DistinguishedName(guarantor));
  }

  /**
    * Constructs this principal.
    *
    * @param name principal's distinguished name
    * @param guarantor ca's distinguished name 
    */
  public AkentiPrincipal(DistinguishedName name, DistinguishedName guarantor) {
    if (name == null || guarantor == null) {
      throw new IllegalArgumentException();
    }

    this.name = name;
    this.guarantor = guarantor;
  }

  /**
    * Constructs this principal.
    *
    * @param source stream used to initialize this object.
    */
  public 
  AkentiPrincipal(AkentiInputStream source) throws IOException, ParseException {
    readObject(source);
  }

  /**
    * Constructs this principal.
    *
    * @param element XML element used to parse this object.
    */
  public 
  AkentiPrincipal(Element element) throws ParseException {
    fromXML(element);
  }

  /**
    * Tests for equality. Useful in containers such as vectors or sets.
    */
  public boolean equals(Object obj) {
    if (obj.getClass().equals(this.getClass())) {
      AkentiPrincipal temp = (AkentiPrincipal) obj;

      return name.equals(temp.name) && guarantor.equals(temp.guarantor);
    }

    return false;
  }
  
  /**
    * Computes a hash that can be used by hash containers such as 
    * a Hashtable.
    */
  public int hashCode() {
    StringBuffer sb =  new StringBuffer(getName().to500String());

    sb.append(getGuarantor().to500String());
    return sb.toString().hashCode();
  }

  /**
    * Returns this principal's distinguished name.
    */
  public DistinguishedName getName() {
    return name;
  }

  /**
    * Returns the guarantor's name.
    */
  public DistinguishedName getGuarantor() {
    return guarantor;
  }

  /**
    * Writes this principal to a <code>AkentiOutputStream</code>.
    */
  public void 
  writeObject(AkentiOutputStream aos) throws IOException {
    name.writeObject(aos);
    aos.write(' ');
    guarantor.writeObject(aos);
  }

  /**
    * Initializes this principal from a <code>AkentiInputStream</code>.
    */
  private void 
  readObject(AkentiInputStream ais) throws IOException, ParseException {
    name = new DistinguishedName(ais);
    guarantor = new DistinguishedName(ais);
  }

  /**
    * Populates the XML element <code>ap</code>. 
    */
  public void toXML(Element ap) {
    Document doc = ap.getOwnerDocument();
    Element temp = doc.createElement("UserDN");

    name.toXML(temp);
    ap.appendChild(temp);
    temp = doc.createElement("CADN");
    guarantor.toXML((Element) temp);
    ap.appendChild(temp);
  }

  /**
    * Initializes this principal from the XML element <code>ap</code>. 
    */
  private void fromXML(Element ap) throws ParseException {
    Node child = ap.getFirstChild();

    if (child == null || child.getNodeType() != Node.ELEMENT_NODE) {
      throw new ParsingException();
    }

    name = new DistinguishedName((Element) child);
    child = child.getNextSibling();

    if (child == null || child.getNodeType() != Node.ELEMENT_NODE) {
      throw new ParsingException();
    }

    guarantor = new DistinguishedName((Element) child);
  }

  /**
    * Returns a string representation of this object that can be 
    * used by gui object.
    */
  public String toString() {
    StringBuffer sb = new StringBuffer();

    sb.append('[');
    sb.append(name.getValue("cn"));
    sb.append(',');
    sb.append(guarantor.getValue("cn"));
    sb.append(']');
    return sb.toString();
  }

  /**
    * Used for debugging.
    */
  public String paramString() {
    StringBuffer sbuf = new StringBuffer();

    sbuf.append("[");
    sbuf.append("name");
    sbuf.append(":");
    sbuf.append(getName().paramString());
    sbuf.append(",");
    sbuf.append("ca");
    sbuf.append(":");
    sbuf.append(getGuarantor().paramString());
    sbuf.append("]");
    return sbuf.toString();
  }
}
