package lbl.akenti.certs.elements;

import java.io.IOException;
import java.text.ParseException;
import lbl.akenti.util.ParsingException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
  * A <code>UseCondIssuerGroup</code> object consists of one more
  * <code>AkentiPrincipal</code> who represent Use-Condition Certificate Issuers
  * and one or more locators that are used to locate Use-Condition certificates
  * issued by those issuers.
  *
  * <pre>
  *
  * UseCondIssuerGroup {
  *   vector:ltAkentiPrincipal&gt; issuers; // UC certificate issuers 
  *   vector&lt;Directory&gt;         ucDirs; // where to find these uc certs 
  * }
  *
  * The format of a UseCondIssuerGroup is:
  *   issuerCount issuer+ ucDirCount ucDir+
  * </pre>
  *
  * @see AkentiObject
  * @see AkentiPrincipal
  * @see Directory
  *
  * @version 1.1 02/05/01
  * @author Srilekha Mudumbai
  * @author Abdelilah Essiari
  */
public final class UseCondIssuerGroup extends AkentiObject {

  private AkentiPrincipal[] issuers;
  private Directory[] ucDirs;

  /**
    * Constructs this object using default values.
    */
  public UseCondIssuerGroup() {
    this(
      new AkentiPrincipal[] { new AkentiPrincipal() },
      new Directory[] { new Directory() }
    ); 
  }

  /**
    * Constructs this <code>UseCondIssuerGroup</code>.
    *
    */
  public UseCondIssuerGroup(AkentiPrincipal[] issuers, Directory[] ucDirs) {
    if (issuers.length == 0 || ucDirs.length == 0) {
      throw new IllegalArgumentException();
    }

    this.issuers = issuers;
    this.ucDirs = ucDirs;
  }

  /**
    * Constructs this <code>UseCondIssuerGroup</code>.
    *
    * @param source stream used to initialize this object.
    */
  public
  UseCondIssuerGroup(AkentiInputStream source) throws IOException, ParseException {
    readObject(source);
  }

  /**
    * Constructs this <code>UseCondIssuerGroup</code>.
    *
    * @param element XML element used to parse this object.
    */
  public
  UseCondIssuerGroup(Element element) throws ParseException {
    fromXML(element);
  }

  /**
    * Returns the <code>UC</code> certificate issuers.
    */
  public AkentiPrincipal[] getUCIssuers() {
    return issuers;
  }

  /**
    * Returns the <code>UC/i> certificate locators.
    */
  public Directory[] getDirectories() {
    return ucDirs;
  }

  /**
    * Returns <i>true</i> if <code>issuer</code> is listed as one of the 
    * <code>UC</code> certificate issuers.
    */
  public boolean containsUCIssuer(AkentiPrincipal issuer) {
    for (int i = 0; i < issuers.length; ++i) {
      if (issuers[i].equals(issuer)) {
        return true;
      }
    }

    return false;
  }

  /**
    * Writes this <code>UseCondIssuerGroup</code> to a <code>AkentiOutputStream</code>.
    */
  public void writeObject(AkentiOutputStream drain) throws IOException {
    drain.write(issuers.length);

    for (int i=0; i < issuers.length; i++) {
      drain.write(' ');
      issuers[i].writeObject(drain);
    }

    drain.write(' ');
    drain.write(ucDirs.length);

    for (int i=0; i < ucDirs.length; i++) {
      drain.write(' ');
      ucDirs[i].writeObject(drain);
    }
  }

  /**
    * Initializes this principal from a <code>AkentiInputStream</code>.
    */
  private void 
  readObject(AkentiInputStream source) throws IOException, ParseException {
    int size = source.readInt();
    
    if (size <= 0) {
      throw new ParsingException("issuers.size = " + size);
    }

    issuers = new AkentiPrincipal[size];

    for (int i = 0; i < size; i++) {
      issuers[i] = new AkentiPrincipal(source);
    }

    size = source.readInt();

    if (size <= 0) {
      throw new ParsingException("ucDirs.size = " + size);
    }

    ucDirs = new Directory[size];

    for (int i = 0; i < size; i++) {
      ucDirs[i] = new Directory(source);
    }
  }

  /**
    * Populates the XML element <code>ucGroup</code>
    */
  public void toXML(Element ucGroup) {
    Document doc = ucGroup.getOwnerDocument();

    for (int i= 0; i < issuers.length; i++) {
      Element temp = doc.createElement("Principal");

      issuers[i].toXML(temp);
      ucGroup.appendChild(temp);
    }

    for (int i=0; i < ucDirs.length; i++) {
      Element temp = doc.createElement("URL");

      ucDirs[i].toXML(temp);
      ucGroup.appendChild(temp);
    }
  }

  /**
    * Initializes this <code>UseCondIssuerGroup</code> from the XML element 
    * <code>ucGroup</code>.
    */
  private void fromXML(Element ucGroup) throws ParseException {
    NodeList list = ucGroup.getElementsByTagName("Principal");
    int len = list.getLength();
    
    if (len == 0) {
      throw new ParsingException("no uc issuers found");
    }

    issuers = new AkentiPrincipal[len];
  
    for (int i=0; i < len; i++) {
      issuers[i] = new AkentiPrincipal((Element) list.item(i));
    }

    list = ucGroup.getElementsByTagName("URL");

    if ((len  = list.getLength()) == 0) {
      throw new ParsingException("no uc dirs found");
    }

    ucDirs = new Directory[len];
  
    for (int i=0;  i < len; i++) {
      ucDirs[i] = new Directory((Element) list.item(i));
    }
  }

  /**
    * Returns debugging info.
    */
  public String paramString() {
    StringBuffer sbuf = new StringBuffer();

    sbuf.append("[");
    sbuf.append(issuers.length);

    for (int i=0; i < issuers.length; i++) {
      sbuf.append(",");
      sbuf.append(issuers[i].paramString());
    }

    sbuf.append(",");
    sbuf.append(ucDirs.length);

    for (int i=0; i < ucDirs.length; i++) {
      sbuf.append(",");
      sbuf.append(ucDirs[i].paramString());
    }

    sbuf.append("]");
    return sbuf.toString();
  }
}
