package lbl.akenti.dirServices;

import java.net.URL;
import java.security.cert.X509Certificate;
import java.util.Hashtable;
import java.util.Map;
import lbl.akenti.certs.AkentiCertificate;

/**
  * Class <code>CertificateRetrievalFactory</code> provides 
  * applications with a convenient way of retrieving
  * certificates from public directories.
  * 
  * @author  Abdelilah Essiari
  *
  * @see CertificateRetrieval
  * @see lbl.akenti.certs.AkentiCertificate
  */
public class CertificateRetrievalFactory {
  private static CertificateRetrievalFactory fac;
  private Map map;

  /**
    * Construct this <code>CertificateRetrievalFactory</code>.
    *
    * Note: only one instance will be created.
    */ 
  private CertificateRetrievalFactory() {
    map = new Hashtable();
  }

  /**
    * Registers a <code>CertificateRetrieval</code> with this factory.
    *
    * @return a previously installed <code>CertificateRetrieval</code> object 
    *         or <code>null</code>.
    */
  public CertificateRetrieval 
  addCertificateRetrieval(String protocol, CertificateRetrieval cr) {
    return (CertificateRetrieval) map.put(protocol, cr);
  }

  /**
    * Returns a <code>X509Certificate</code> associated with 
    * the given protocol. 
    */
  public CertificateRetrieval getCertificateRetrieval(String protocol) {
    return (CertificateRetrieval) map.get(protocol);
  }

  /**
    * Returns the only instance that will ever be created.
    */
  public static CertificateRetrievalFactory getInstance() {
    if (fac == null) {
      synchronized(CertificateRetrievalFactory.class) {
        if (fac == null) {
          fac = new CertificateRetrievalFactory();
          fac.addCertificateRetrieval("file", new FileCertificateRetrieval());
          fac.addCertificateRetrieval("http", new HttpCertificateRetrieval());
          fac.addCertificateRetrieval("ldap", new LdapCertificateRetrieval());
        }
      }
    }

    return fac;
  }

  /**
    * Convenient way of retrieving <code>X509Certificate</code>s.
    */
  public static X509Certificate[] retrieveX509Certificates(URL url, Map args) {
    CertificateRetrieval cr = getInstance().getCertificateRetrieval(url.getProtocol());
    
    if (cr == null) {
      // LOG THIS?
      return new X509Certificate[0];
    }

    return cr.retrieveX509Certificates(url, args);
  }

  /**
    * Convenient way of retrieving <code>AkentiCertificate</code>s.
    */
  public static AkentiCertificate[] retrieveAkentiCertificates(URL url, Map args)  {
    CertificateRetrieval cr = getInstance().getCertificateRetrieval(url.getProtocol());

    if (cr == null) {
      // LOG THIS?
      return new AkentiCertificate[0];
    }

    return cr.retrieveAkentiCertificates(url, args);
  }
}

