/**
  * A simple SSLv3 client that writes "hello" to the echo server.
  * 
  * This client has a p12 file that stores its private key and its 
  * chain of certificates.
  *
  * @author Abdelilah Essiari
  */

package lbl.akenti.examples.SSLServer;

import java.io.BufferedInputStream;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.Socket;
import java.security.KeyManagementException;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.UnrecoverableKeyException;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;
import javax.net.SocketFactory;
import javax.net.ssl.KeyManagerFactory;
import javax.net.ssl.SSLContext;
import javax.net.ssl.TrustManagerFactory;

public class TestClient {
    /**
      * Loads p12 file into key store.
      * Loads trusted cas into key store.
      * Intializes the context.
      * Creates Socket
      * Writes "hello"
      * Reads echoed string (Hello)
      */
    public static void main(String args[]) {
      try {

        /**
          * <i>p12FileName</i> holds client's key and chain.
          * <i>casFileName</i> holds tusted cas' x509 certs.
          * <i>password</i> locks the p12 file.
          * <i>host</i> where server is running.
          * <i>port</i> at which the server is listening. 
          *
          */
        String p12FileName = "client.p12";
        String casFileName = "cas.pem";
        char[] password = "testing".toCharArray();

        int port = 9999;
    
        if (args.length != 1) {
           System.out.println("usage: java TestClient <host-name> "); 
           System.exit(1);
        } 
   
        String host = args[0];

        KeyStore ks = KeyStore.getInstance("pkcs12");

        // Loads p12 file
        ks.load(new FileInputStream(p12FileName), password);

        // Adds trusted ca certs to key store.
        // It is important to use a stream that supports marks.
        CertificateFactory cfac = CertificateFactory.getInstance("X.509");
        BufferedInputStream bis = new BufferedInputStream(
          new FileInputStream(casFileName)
        );

        do {
          // We use the dn for an alias
          X509Certificate cert = (X509Certificate) cfac.generateCertificate(bis);
          String alias = cert.getSubjectDN().toString();

          ks.setCertificateEntry(alias, cert);
        } while (bis.available() > 0);

        KeyManagerFactory kfac = KeyManagerFactory.getInstance("SunX509");

        kfac.init(ks, password);

        TrustManagerFactory tfac = TrustManagerFactory.getInstance("SunX509");

        tfac.init(ks);
       
        // We use SSLv3.
        SSLContext ctx = SSLContext.getInstance("SSLv3");

        // Initializes the context.
        // U can always save the random number
        // to get things going faster.
       
        System.out.println("this will take some time...");

        ctx.init(
          kfac.getKeyManagers(),   // Used to authenticate to server.
          tfac.getTrustManagers(), // Used to authenticate the server.
          null                     // Here u pass in a random
        );

        // Gets a factory that knows how to create SSL sockets.
        SocketFactory factory = ctx.getSocketFactory();

        // From here on we do the usual stuff
        // Hard to tell we are using an ssl socket
        // but we are.
        Socket sock = factory.createSocket(host, port);
        DataOutputStream out = new DataOutputStream(sock.getOutputStream());
        String str = "hello";

        out.writeInt(str.length());
        out.writeBytes(str);
        out.flush();

        DataInputStream in = new DataInputStream(sock.getInputStream());
        int len = in.readInt();
        byte[] buffer = new byte[len];

        in.readFully(buffer, 0, len);

        if (str.equals(new String(buffer))) {
          System.out.println("JUST TALKED TO ECHO SERVER");
        } else {
          System.out.println("THE SERVER FAILED TO ECHO");
        }

        sock.close();
      } catch (KeyManagementException  e) {
        e.printStackTrace();
      } catch (UnrecoverableKeyException e) {
        e.printStackTrace();
      } catch (NoSuchAlgorithmException e) {
        e.printStackTrace();   
      } catch (KeyStoreException e) {
        e.printStackTrace();
      } catch (FileNotFoundException e) {
        e.printStackTrace(); 
      } catch (IOException e) {
        e.printStackTrace();
      } catch (CertificateException e) {
        e.printStackTrace();
      }
    }
}
