package lbl.akenti.gen.controller;

import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.JTextArea;
import javax.swing.JPasswordField;
import javax.swing.JList;
import javax.swing.ListSelectionModel;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.WindowConstants;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.ListSelectionEvent;
import java.awt.Component;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.BorderLayout;

/**
 * LBLDialogFactory.java
 *
 * Two flavors of dialogs are created by this class.
 *
 *    simple info dialogs
 *
 *    simple input dialogs  
 *  
 * The methods showInputDialog(...) can be used to show dialogs when no 
 * handle on the Dialog is needed. If a handle is needed then use 
 * createInputDialog.
 *
 * @author Abdelilah Essiari
 */
public class LBLDialogFactory {

  /**
    * Disallow contruction of this object. 
    */
  private LBLDialogFactory() { }

  /**
    * Create an Info Dialog with no title.
    * <p>
    * @param owner       parent frame
    * @param message     message to prompt user
    * @return            a <code>JDialog</code> 
    * @see               LBLDialogFactory
    */
  public static JDialog createInfoDialog(Component owner, String message) {
    return createInfoDialog(owner, null, message);
  }

  /**
   * Create an Info Dialog with title
   * <p> 
   *    Usage:
   *    <p>
   *    JDialog d = LBLDialogFactory.createInfoDialog(null, 
   *                                   "Fatal Error", 
   *                                   "Computer on fire. Press OK To Exit");
   *    <p>
   *    d.setModal(false)   --  modal set to true by default <br>
   *    d.setVisible(true); -- if modal is true, this call blocks until the user clicks OK<br>
   * <p>
   * @param owner       parent frame
   * @param title       the title string for the dialog
   * @param message     message to prompt user
   * @return            a <code>JDialog</code>
   * @see               JOptionPane
   */
  public static JDialog createInfoDialog(Component owner, String title, String message) {
    JOptionPane pane = new JOptionPane(createHeader(message));

    return pane.createDialog(owner, title);
  }

  /**
    */
  public static void showInfoDialog(Component owner, String message) {
    showInfoDialog(owner, null, message);
  }

  /**
    */
  public static void showInfoDialog(Component owner, String title, String message) {
    createInfoDialog(owner, title, message).setVisible(true);
  }


  /**
   * Display an input dialog with no title and with a <code>JTextField</code>
   * Used to accept user's input. 
   * <p>
   * @param owner       parent frame
   * @param message     message to prompt user
   * @return            the text entered by user
   * @see               LBLDialogFactory
   */
  public static String showInputDialog(Component owner, String message) {
    return showInputDialog(owner, null, message);
  }

  /**
   * Display an input dialog with a title and a <code>JTextField</code>
   *  used to accept user's input. 
   * <p>
   * @param owner       Parent frame
   * @param title       the title string for the dialog
   * @param message     message to prompt user
   * @return            the text entered by user
   * @see               LBLDialogFactory
   */
  public static String showInputDialog(Component owner, String title, String message) {
    JTextField field = new JTextField();
    JOptionPane pane = createOptionPane(message, field);

    pane.createDialog(owner, title).setVisible(true);

    Object val = pane.getValue();

    if (val != null && val.equals(new Integer(JOptionPane.OK_OPTION))) {
      return field.getText();
    }

    return null;
  }

  public static void showText(Component owner, String title, String message, String text) {
    JPanel panel = new JPanel(new BorderLayout(5, 5));

    if (message != null) {
      panel.add(createHeader(message), "North");
    }

    JTextArea area = new JTextArea(10, 10);

    panel.add(new JScrollPane(area), "Center");

    area.setText(text);
    new JOptionPane(panel).createDialog(owner, title).setVisible(true);
  }

  /**
   * Display password dialog with a title. The text the user types in will
   * not be echoed to the screen.
   *
   * @param owner       frame who owns dialog  
   * @param title       the title string for the dialog, if not null 
   * @param message     message to prompt user
   * @return            null if user cancels or closes the dialog, 
   *                    the text entered by user otherwise.
   * @see               LBLDialogFactory
   * @see               JOptionPane
   */
  public static String showPasswordDialog(Component owner, String title, String message) {
    JTextField field =  new JPasswordField();
    JOptionPane pane = createOptionPane(message, field);
    
    pane.createDialog(owner, title).setVisible(true);

    Object val = pane.getValue();
   
    if (val != null && val.equals(new Integer(JOptionPane.OK_OPTION))) {
      return field.getText();
    }

    return null;
  }

  
  /**
   * Create a <code>JOptionPane</code>.
   * <p>
   * @param message     message to prompt user
   * @param field       used to accept user input
   * @return            a <code>JOptionPane</code>
   * @see               JOptionPane 
   * @see               AKTextCanvas
   */
  private static JOptionPane 
  createOptionPane(String message, JTextField field) {
    JPanel panel = new JPanel(new BorderLayout(5, 5));

    panel.add(createHeader(message), "North");
    panel.add(field, "South");

    JOptionPane pane = new JOptionPane(panel);

    pane.setOptionType(JOptionPane.OK_CANCEL_OPTION);
    return pane;
  }
  
  public static Object[] 
  showListDialog(Component owner, String title, String message, Object[] inputs) {
    return (Object[]) showListDialog(
      owner, 
      title, 
      message, 
      inputs, 
      ListSelectionModel.MULTIPLE_INTERVAL_SELECTION
    );
  }

  /**
    */
  public static Object 
  showListDialog(Component owner, String title, String message, Object[] inputs, int selectionMode) {
    final JOptionPane pane = new JOptionPane();
    final JList list = new JList(inputs);

    list.setVisibleRowCount(5);
    list.setSelectionMode(selectionMode);

    pane.setLayout(new BorderLayout(5, 5));
    pane.add(createHeader(message), "North");
    pane.add(new JScrollPane(list), "Center");
    
    JPanel buttonPanel = new JPanel();

    ActionListener actionListener = new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        if (e.getActionCommand().equals("OK")) {
          if (list.getSelectionMode() == ListSelectionModel.SINGLE_SELECTION) {
            pane.setValue(list.getSelectedValue());
          } else {
            pane.setValue(list.getSelectedValues());
          }
        } else {   // CANCEL
          pane.setValue(GenConsts.AK_CANCEL_VALUE);
        }
      }
    };

    final JButton okButton = new JButton("OK");
    JButton cancelButton = new JButton("CANCEL");

    okButton.addActionListener(actionListener);
    okButton.setVisible(false);
    cancelButton.addActionListener(actionListener);
    buttonPanel.add(okButton);
    buttonPanel.add(cancelButton);
    pane.add(buttonPanel, "South");
    list.addListSelectionListener(new ListSelectionListener() {
      public void valueChanged(ListSelectionEvent e) {
        if (!e.getValueIsAdjusting()) {
          boolean visible = list.getSelectedIndex() != -1;

          okButton.setVisible(visible);
        }
      }
    });

    pane.createDialog(owner, title).setVisible(true);
  
    Object ret = pane.getValue();

    if (ret == GenConsts.AK_CANCEL_VALUE) {
      return null;
    }

    return ret;
  }

  public static boolean 
  showDialog(Component owner, String title, String message, JComponent comp) {
    JPanel panel = new JPanel(new BorderLayout(5, 5));

    panel.add(createHeader(message), "North");
    panel.add(comp, "Center");

    JOptionPane pane = new JOptionPane(panel);

    pane.setOptionType(JOptionPane.OK_CANCEL_OPTION);
    pane.createDialog(owner, title).setVisible(true);

    Object val = pane.getValue();

    if (val != null && val.equals(new Integer(JOptionPane.OK_OPTION))) {
      return true;
    }

    return false;
  }
  
  private static JComponent createHeader(String message) {
    if (message.startsWith("<html>")) {
      return LBLLabelFactory.createHtmlLabel(message);
    } else {
      return new AKTxtCanvas(message);
    }
  }
}
