package lbl.akenti.messaging;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.Writer;
import java.security.cert.CertificateException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import javax.xml.bind.JAXBException;
import lbl.akenti.jaxb.AkentiRequestType;
import lbl.akenti.jaxb.AkentiRequestsGenerated;
import lbl.akenti.jaxb.CheckAccessRequestType;
import lbl.akenti.jaxb.CheckPolicyRequestType;
import lbl.akenti.jaxb.CreateResourceRequestType;
import lbl.akenti.jaxb.GetResourceInfoRequestType;
import lbl.akenti.jaxb.ResourceBrowseRequestType;
import lbl.akenti.jaxb.UploadPolicyCertificateRequestType;
import org.w3c.dom.Element;

/**
  * Represents an immutable non-empty list of <code>AkentiRequest</code>s that
  * are handled by the Akenti server.                            <BR>
  *
  * Currently The server handles the following requests:         <BR>
  *
  * The server handles the following requests:                   <BR>
  *
  * CheckAccessRequest:                                          <BR>
  *   Used when determining a user's access to a resource.       <BR>
  *
  * CheckPolicyRequest:                                          <BR>
  *   Used by stakholders to debug policy                        <BR>
  *
  * ResourceBrowseRequest:                                       <BR>
  *   Used to determine daughter resources of given resource     <BR>
  *
  * CreateResourceRequest:                                       <BR>
  *   Used to create resources                                   <BR>
  *
  * UploadPolicyCertificateRequest:                              <BR>
  *   Used to Upload a policy certificate                        <BR>
  *
  * GetResoourceInfoRequest:                                     <BR>
  *   Used to upload a resource attrs/hints                      <BR>
  *
  * Each request contains a client-generated unique id that can be useful 
  * when clients are sending or keeping track of multiple requests.
  * 
  * @author Abdelilah Essiari.
  */

final public 
class AkentiRequests {
  private List requests;

 /**
   * Constructs this <code>AkentiRequests</code>.
   * @param ar a single AkentiRequest.
   */
  public AkentiRequests(AkentiRequest ar) {
    this(new AkentiRequest[]{ar});
  }

 /**
   * Constructs this <code>AkentiRequests</code>.
   * @param ars an array of AkentiRequest of size at least 1.
   */
  public AkentiRequests(AkentiRequest[] ars) {
    requests = new ArrayList(ars.length);
    requests.add(ars[0]);
   
    for (int i = 1; i < ars.length; ++i) {
      requests.add(ars[i]);
    } 
    
    requests = Collections.unmodifiableList(requests);
  }

 /**
   * Constructs this <code>AkentiRequests</code> from an input stream
   * containing <code>XML</code> data.
   * @param in an InputStream constructed with a streamized Requests
   * @throws ParseException if the stream does not define a valid requests
   * @throws CertificateException if unsupported encoding type is used
   * @throws IOException when unable to read this object
   */
  public AkentiRequests(InputStream in) throws IOException, ParseException, CertificateException {
    try {
      init((AkentiRequestsGenerated) Util.getUnmarshaller().unmarshal(in));
    } catch(JAXBException ex) {
      throw new IOException(ex.getMessage());
    }
  }

 /**
   * Constructs this <code>AkentiRequests</code> from a <code>DOM</code> element.
   * @param element an element containing an AkentiReqeusts
   * @throws ParseException if the stream does not define a valid AkentiRequest
   * @throws CertificateException if unsupported encoding type is used
   * @throws IOException when unable to build this object
   */
  public AkentiRequests(Element element) throws IOException, ParseException, CertificateException {
    try {
      init((AkentiRequestsGenerated) Util.getUnmarshaller().unmarshal(element));
    } catch(JAXBException ex) {
      throw new ParseException(ex.getMessage(), -1);
    }
  }

 /**
   * Returns the requests as an iterator.
   * @see AkentiRequest
   * @return the requests iterator
   */
  public Iterator getAkentiRequests() {
    return requests.listIterator();
  }
  
 /**
   * Returns the requests as an array.
   * @see AkentiRequest
   * @return the request array
   */
  public AkentiRequest[] toAkentiRequestsArray() {
    AkentiRequest[] ret = new AkentiRequest[requests.size()];

    requests.toArray(ret);
    return ret;
  }

 /**
   * Turns this request into a stream. To reconstitute the request
   * use the constructor that takes an InputStream.
   * @param out the request as a stream
   * @throws IOException if the request cannot be streamed
   * @see AkentiRequest
   */
  public void toStream(OutputStream out) throws IOException {
    try {
      Util.getMarshaller().marshal(toJAXBImpl(), out);
    } catch(JAXBException ex) {
      throw new IOException(ex.getMessage());
    }
  }

 /**
   * Marshals this requests's data into the given writer.
   * @param writer the writer
   * @throws IOException if there is an problem marshalling the data
   */
  public void toWriter(Writer writer) throws IOException {
    try {
      Util.getMarshaller().marshal(toJAXBImpl(), writer);
    } catch(JAXBException ex) {
      throw new IOException(ex.getMessage());
    }
  }

 /**
   * Marshals this request's data into the given element.
   * @param element the element which will contain the data.
   * @throws ParseException if there is an problem marshalling the data
   */
  public void toElement(Element element) throws ParseException {
    try {
      Util.getMarshaller().marshal(toJAXBImpl(), element);
    } catch(JAXBException ex) {
      throw new ParseException(ex.getMessage(), -1);
    }
  }

 /**
    * FOR XML STUFF
    */
  private void init(AkentiRequestsGenerated impl) throws IOException, ParseException, CertificateException {
    List temp = impl.getAkentiRequest();
    Iterator iter = temp.listIterator();

    requests = new ArrayList(temp.size());

    do {
      AkentiRequestType req = (AkentiRequestType) iter.next();

      if (req instanceof CheckAccessRequestType) {
        requests.add(new CheckAccessRequest((CheckAccessRequestType) req));
      } else if (req instanceof CheckPolicyRequestType) { 
        requests.add(new CheckPolicyRequest((CheckPolicyRequestType) req));
      } else if (req instanceof ResourceBrowseRequestType) { 
        requests.add(new ResourceBrowseRequest((ResourceBrowseRequestType) req));
      } else if (req instanceof CreateResourceRequestType) { 
        requests.add(new CreateResourceRequest((CreateResourceRequestType) req));
      } else if (req instanceof UploadPolicyCertificateRequestType) { 
        requests.add(new UploadPolicyCertificateRequest((UploadPolicyCertificateRequestType) req));
      } else if (req instanceof GetResourceInfoRequestType) { 
        requests.add(new GetResourceInfoRequest((GetResourceInfoRequestType) req));
      } else {
        throw new RuntimeException();
      }
    } while (iter.hasNext());

    requests = Collections.unmodifiableList(requests);
  }

 /** 
    * FOR XML STUFF
    */
  private AkentiRequestsGenerated toJAXBImpl() throws JAXBException {
    AkentiRequestsGenerated impl = Util.getObjectFactory().createAkentiRequestsGenerated();
    Iterator iter = getAkentiRequests();
    List temp = impl.getAkentiRequest();

    do {
      AkentiRequest ar = (AkentiRequest) iter.next();

      temp.add(ar.toJAXBImpl(false));
    } while (iter.hasNext());

    return impl;
  }
}
