package lbl.akenti.messaging;

import java.io.IOException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import javax.xml.bind.JAXBException;
import lbl.akenti.certs.AkentiCertificate;
import lbl.akenti.certs.PolicyCertificate;
import lbl.akenti.certs.UseCondCertificate;
import lbl.akenti.jaxb.AkentiMsgType;
import lbl.akenti.jaxb.CertificateInfoType;

/**
 * Consists of an <code>AkentiCertificate</code> and an optional <code>AkentiMessage</code>
 * that describes whether the certificate was used or rejected by the <code>AkentiEngine</code>
 *
 * @author Abdelilah Essairi.
 */

public
class CertificateInfo {
  private AkentiCertificate cert;
  private List mesgsList;
  private boolean used;
  private String location;
  
 /**
   * Constructs this <code>CertificateInfo</code> with a message.
   * @see AkentiCertificate
   * @param cert the certificate
   * @param mesgs messages describing the cert
   * @param location the location
   * @param used whether this CertificateInfo is used
   */
  public CertificateInfo(AkentiCertificate cert, AkentiMessage[] mesgs, String location, boolean used) {
    if (cert == null || mesgs == null || location == null) {
      throw new IllegalArgumentException();
    }
    
    this.cert = cert;
    this.location = location;
    this.used = used;
    
    mesgsList = new ArrayList(mesgs.length);
    for (int i = 0; i < mesgs.length; ++i) {
      mesgsList.add(mesgs[i]);
    }
      
      mesgsList = Collections.unmodifiableList(mesgsList);
  }
  
 /**
   * Returns the <code>AkentiCertificate</code>.
   * @see AkentiCertificate
   * @return the cert
   */
  public AkentiCertificate getAkentiCertificate() {
    return cert;
  }
  
 /**
   * Returns the used attribute.
   * @return whether this cert is used.
   */
  public boolean isUsed() {
    return used;
  }

 /**
   * Returns the location.
   * @return the location
   */
  public String getLocation() {
    return location;
  }
  
 /**
   * Returns <i>true</i> if at least one <code>AkentiMessage</code> is present.
   * @see #getAkentiMessages()
   * @return whether this has messagesor not
   */
  public boolean hasAkentiMessages() {
    return mesgsList != null;
  }
  
 /**
   * Returns the <code>AkentiMessage</code>s.
   * @see #hasAkentiMessages()
   * @return the messages
   */
  public AkentiMessage[] getAkentiMessages() {
    return (AkentiMessage[]) mesgsList.toArray(new AkentiMessage[mesgsList.size()]);
  }
  
 /**
   * Returns a string with basic information; used by GUI Display tool
   * @return this object in string form
   */
  public String toString() {
    if (cert.getType().equals("UseCondition")) {
      return "UseCondition for " + ((UseCondCertificate) cert).getResource();
    } else if (cert.getType().equals("Policy")) {
      return "Policy for " + ((PolicyCertificate) cert).getResource();
    } else {
      return "Undefined toString for certificate type";
    }
  }
  
 /**
   * Constructs this <code>CertificateInfo</code> with a message.
   * @param impl the CertificateInfoType
   * @throws IOException if the encoded x509 certificate is flawed
   * @throws ParseException if there is an error parsing the data
   */
  CertificateInfo(CertificateInfoType impl)
  throws IOException, ParseException {
    if (impl.getOpaqueCertificate() != null) {
      cert = OpaqueCertificate.fromJAXBImpl(impl.getOpaqueCertificate());
    }
    
    List tempList = impl.getCertificateMessage();
    location = impl.getLocation();
    used = impl.isUsed();
    
    if (tempList != null) {
      Iterator iter = tempList.iterator();
      
      mesgsList = new ArrayList(tempList.size());
      
      while (iter.hasNext()) {
        AkentiMsgType art = (AkentiMsgType) iter.next();
        mesgsList.add(new AkentiMessage(art));
      }
      
      mesgsList = Collections.unmodifiableList(mesgsList);
    }
  }
  
 /**
   * Produces a CCertificateInfoType to encapsulate the data in this object.
   * @throws JAXBException if mandatory data is not present
   * @return the CertificateInfoType
   */
  CertificateInfoType toJAXBImpl() throws JAXBException {
    CertificateInfoType impl = Util.getObjectFactory().createCertificateInfoType();
    
    impl.setOpaqueCertificate(OpaqueCertificate.toJAXBImpl(cert));
    impl.setUsed(used);
    impl.setLocation(location);
    
    if (hasAkentiMessages()) {
      List tempList = impl.getCertificateMessage();
      Iterator iter = mesgsList.iterator();
      
      while (iter.hasNext()) {
        AkentiMessage art = (AkentiMessage) iter.next();
        tempList.add(art.toJAXBImpl());
      }
            
    }
    
    return impl;
  }
}