package lbl.akenti.messaging;

import java.io.ByteArrayInputStream;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import javax.xml.bind.JAXBException;
import lbl.akenti.certs.elements.AkentiPrincipal;
import lbl.akenti.jaxb.AkentiRequestType;
import lbl.akenti.jaxb.CheckAccessRequestType;
import lbl.akenti.jaxb.PrincipalType;
import lbl.akenti.jaxb.RestrictedAttributeType;
import lbl.akenti.jaxb.X509CertificateChainType;

/**
 * Holds the minimum inputs to the <code> Akenti Authorization Engine</code>.
 * These are an entity's DN/CADN pair and the name of the resource
 * the entity is trying to access. Other inputs will be added in the
 * future.
 *
 * @author Abdelilah Essiari
 **/

final public
class CheckAccessRequest extends AkentiRequest {
  private String resource;
  private AkentiPrincipal ap;
  private List x509, resAttrs, desActs;
  private boolean isDetailed;
  
 /**
   * Constructs a <code>CheckAccessRequest</code>.
   * @see AkentiPrincipal
   * @param uid a unique id
   * @param ap the DN/CADN pair of an entity.
   * @param resource the resource
   */
  
  public CheckAccessRequest(int uid, AkentiPrincipal ap, String resource) {
    super(uid);
    if (ap == null || resource == null || resource.length() == 0) {
      throw new IllegalArgumentException();
    }
    
    this.ap = ap;
    this.resource = resource;
    initOptional(null,null,false);
  }
  
 /**
   * Constructs a <code>CheckAccessRequest</code>.
   * @param uid the unique id
   * @param x509 the x509 of the user in question
   * @param resource the resource
   */  
  public CheckAccessRequest(int uid, X509Certificate[] x509, String resource) {
    super(uid);
    
    if (x509 == null || resource == null || resource.length() == 0) {
      throw new IllegalArgumentException();
    }
    
    List x509Temp = new ArrayList(x509.length);
    for (int i = 0; i < x509.length; ++i) {
      x509Temp.add(x509[i]);
    }
    this.x509 = Collections.unmodifiableList(x509Temp);
    this.resource = resource;
    initOptional(null,null,false);
  }
  
  
 /**
   * Constructs a <code>CheckAccessRequest</code>.
   * @param uid the unique id
   * @param ap the DN/CADN pair of an entity.
   * @param resource the resource
   * @param desActs the desired actions
   * @param resAttrs the restricted attributes
   * @param isDetailed whether to return a detailed response
   */  
  public CheckAccessRequest(int uid, AkentiPrincipal ap, String resource,
  String[] desActs, RestrictedAttribute[] resAttrs, boolean isDetailed) {
    super(uid);
    
    if (ap == null || resource == null || resource.length() == 0) {
      throw new IllegalArgumentException();
    }
    
    this.ap = ap;
    this.resource = resource;
    
    initOptional(desActs, resAttrs, isDetailed);
  }
  
 /**
   * Constructs a <code>CheckAccessRequest</code>.
   * @param uid the unique id
   * @param x509 the x509 of the user in question
   * @param resource the resource
   * @param desActs the desired actions
   * @param resAttrs the restricted attributes
   * @param isDetailed whether to return a detailed response
   */  
  public CheckAccessRequest(int uid, X509Certificate[] x509, String resource,
  String[] desActs, RestrictedAttribute[] resAttrs, boolean isDetailed) {
    super(uid);
    
    if (x509 == null || resource == null || resource.length() == 0) {
      throw new IllegalArgumentException();
    }
    
    List x509Temp = new ArrayList(x509.length);
    for (int i = 0; i < x509.length; ++i) {
      x509Temp.add(x509[i]);
    }
    this.x509 = Collections.unmodifiableList(x509Temp);
    
    this.resource = resource;
    
    initOptional(desActs, resAttrs, isDetailed);
  }
  
  private void initOptional(String[] desActs, RestrictedAttribute[] resAttrs, boolean isDetailed){
    if (desActs != null) {
      List desTemp = new ArrayList(desActs.length);
      for (int i = 0; i < desActs.length; ++i) {
        desTemp.add(desActs[i]);
      }
      this.desActs = Collections.unmodifiableList(desTemp);
    } else {
      desActs = null;
    }
    
    if (resAttrs != null) {
      List resTemp = new ArrayList(resAttrs.length);
      for (int i = 0; i < resAttrs.length; ++i) {
        resTemp.add(resAttrs[i]);
      }
      this.resAttrs = Collections.unmodifiableList(resTemp);
    } else {
      resAttrs = null;
    }
    
    this.isDetailed = isDetailed;
  }
  
  
 /**
   * Returns the name of the resource.
   * @return the resource
   */
  public String getResource() {
    return resource;
  }
  
 /**
   * Returns the DN/CADN pair of the entity.
   * @return the AkentiPrincipal of the user in question
   */
  public AkentiPrincipal getAkentiPrincipal() {
    return ap;
  }
  
 /**
   * Returns the X509 Certificate Chain.
   * @return the x509 array
   */
  public X509Certificate[] getX509CertificateChain() {
    if (x509 == null) {
      return null;
    }
    return (X509Certificate[]) x509.toArray(new X509Certificate[x509.size()]);
  }
  
 /**
   * Returns the restricted attributes.
   * @return the restricted Attributes
   */
  public RestrictedAttribute[]  getRestrictedAttributes() {
    if (resAttrs == null) {
      return null;
    }
    return (RestrictedAttribute[]) resAttrs.toArray(new RestrictedAttribute[resAttrs.size()]);
  }
  
 /**
   * Returns the desired actions.
   * @return the desired actions
   */
  public String[] getDesiredActions() {
    if (desActs == null) {
      return null;
    }
    return (String[]) desActs.toArray(new String[desActs.size()]);
  }
  
 /**
   * Returns true if this is detailed.
   * @return if detailed
   */
  public boolean isDetailed() {
    return isDetailed;
  }
  
 /**
   * Constructs a <code>CheckAccessRequest</code> from a
   * <code>CheckAccessRequestType</code>.
   * @param impl the CheckAccessRequestType
   * @throws ParseException if mandatory data is not present
   * @throws CertificateException if the certificate type is incorrect
   */
  CheckAccessRequest(CheckAccessRequestType impl) throws ParseException, CertificateException{
    super(impl);
    resource = impl.getResource();
    
    if (impl.getPrincipal() != null) {
      PrincipalType x = impl.getPrincipal();
      ap = new AkentiPrincipal(x.getUserDN(), x.getCADN());
    } else {
      x509 = new ArrayList();
      List x = impl.getX509CertificateChain();
      CertificateFactory cfac = CertificateFactory.getInstance("X.509");
      for (int i=0; i < x.size(); i++) {
        byte[] bits = ((X509CertificateChainType) x.get(i)).getX509Certificate();
        ByteArrayInputStream bais = new ByteArrayInputStream(bits);
        x509.add((X509Certificate) cfac.generateCertificate(bais));
      }
      this.x509 = Collections.unmodifiableList(x509);
    }
    
    desActs = impl.getDesiredAction();
    
    if (impl.getRestrictedAttribute() != null) {
      resAttrs = new ArrayList();
      List x = impl.getRestrictedAttribute();
      for (int i=0; i < x.size(); i++) {
        RestrictedAttributeType temp = (RestrictedAttributeType) x.get(i);
        resAttrs.add(new RestrictedAttribute(temp));
      }
      this.resAttrs = Collections.unmodifiableList(resAttrs);
    }
    
    isDetailed = impl.isDetailed();
  }
  
 /**
   * Produces a CheckAccessRequestType to encapsulate the data in this object.
   * @param type true uses RequestType false uses RequestGenerated
   * @throws JAXBException if mandatory data is not present
   * @return the CheckAccessRequestType
   */
  protected AkentiRequestType doToJAXBImpl(boolean type) throws JAXBException {
    CheckAccessRequestType impl = type? Util.getObjectFactory().createCheckAccessRequestType()
    : Util.getObjectFactory().createCheckAccessRequestGenerated();
    
    impl.setResource(getResource());
    
    if (getAkentiPrincipal() != null) {
      PrincipalType x = Util.getObjectFactory().createPrincipalType();
      x.setUserDN(ap.getName().to500String());
      x.setCADN(ap.getGuarantor().to500String());
      impl.setPrincipal(x);
    } else {
      ArrayList x = new ArrayList();
      for (int i=0; i < x509.size(); i++) {
        X509CertificateChainType temp = Util.getObjectFactory().createX509CertificateChainType();
        temp.setX509Certificate(((X509Certificate) x509.get(i)).getSignature());
      }
    }
    
    if (desActs != null) {
      impl.getDesiredAction().addAll(desActs);
    }
    
    if (resAttrs != null) {
      ArrayList temp = new ArrayList();
      for (int i=0; i < resAttrs.size(); i++) {
        RestrictedAttributeType x = ((RestrictedAttributeType) resAttrs.get(i));
        temp.add(new RestrictedAttribute(x));
      }
    }
    
    impl.setDetailed(isDetailed);
    
    return impl;
  }
  
}