package lbl.akenti.messaging;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import javax.xml.bind.JAXBException;
import lbl.akenti.certs.AkentiCertificate;
import lbl.akenti.certs.CapabilityCertificate;
import lbl.akenti.certs.elements.AkentiInputStream;
import lbl.akenti.certs.elements.AkentiOutputStream;
import lbl.akenti.jaxb.AkentiResponseType;
import lbl.akenti.jaxb.CheckAccessResponseType;
import lbl.akenti.jaxb.OpaqueCertificateType;
import lbl.akenti.jaxb.PolicyInfoType;
import lbl.akenti.util.Base64;

/**
  * Wraps a <code>CapabilityCertificate</code>. This certificate will  only
  * be present in response to a successful <code>CheckAccessRequest</code>.
  * 
  * @author Abdelilah Essiari
  */
final public 
class CheckAccessResponse extends AkentiResponse {

  private CapabilityCertificate capCert;
  private List polInfos, certs;

 /**
   * Constructs this <code>CheckAccessResponse</code> with no
   * <code> CapabilityCertificate</code>.
   * @param uid the id
   * @param mesg the akentiMessage
   */
  public CheckAccessResponse(int uid, AkentiMessage mesg) {
    this(uid, mesg, null);
  }

 /**
   * Constructs this <code>CheckAccessResponse</code>.
   * @param uid the id
   * @param mesg the akentiMessage
   * @param capCert the Capability Certificate
   */
  public CheckAccessResponse(int uid, AkentiMessage mesg, CapabilityCertificate capCert) {
    super(uid, mesg);
    this.capCert = capCert;
    this.certs = null;
    this.polInfos = null;
  }
    
 /**
   * Constructs this <code>CheckAccessResponse</code>.
   * @param uid the id
   * @param mesg the AkentiMessage
   * @param capCert the CapabilityCertificate
   * @param attrCerts the AkentiCertificates
   * @param polInfos the PolicyInfos
   */
  public CheckAccessResponse(int uid, AkentiMessage mesg, CapabilityCertificate capCert,
    AkentiCertificate[] attrCerts, PolicyInfo[] polInfos) {
    super(uid, mesg);
    this.capCert = capCert;
    
    if (attrCerts != null) {
      ArrayList certTemp = new ArrayList();
      for (int i=0; i < attrCerts.length; i++) {
        certTemp.add(attrCerts[i]);
      }
      this.certs = Collections.unmodifiableList(certTemp);
    }
    
    if (polInfos != null) {
      ArrayList pITemp = new ArrayList();
      for (int i=0; i < attrCerts.length; i++) {
        pITemp.add(attrCerts[i]);
      }
      this.polInfos = Collections.unmodifiableList(pITemp);
    }
  }

 /**
   * Returns <i>true</i> if a <code>CapabilityCertificate</code>
   * is present.
   * @return if has a Capability Certificate
   * @see CapabilityCertificate
   */
  public boolean hasCapabilityCertificate() {
    return capCert != null;
  }

 /**
   * Returns the <code>CapabilityCertificate</code>.
   * @see #hasCapabilityCertificate()
   * @return the CapabilityCertificate
   */
  public CapabilityCertificate getCapabilityCertificate() {
    return capCert;
  }
  
 /**
   * Returns the <code>AkentiCertificate</code>s.
   * @return the AkentiCertificates
   */
  public AkentiCertificate[] getCertificates() {
    if (certs == null) {
      return null;
    }
    return (AkentiCertificate[]) certs.toArray(new AkentiCertificate[certs.size()]);
  }
    
 /**
   * Returns the <code>PolicyInfo</code>s.
   * @return the PolicyInfos
   */
  public PolicyInfo[] getPolicyInfos() {
    if (certs == null) {
      return null;
    }
    return (PolicyInfo[]) polInfos.toArray(new PolicyInfo[polInfos.size()]);
  }

 /**
   * Constructs this <code>CheckAccessResponse</code> from a
   * <code>CheckAccessResponseType</code>.
   * @param impl CheckAccessResponseType
   * @throws IOException if the x509 certificate is not formatted correctly
   * @throws ParseException if mandatory data is not present
   */
  CheckAccessResponse(CheckAccessResponseType impl) 
    throws IOException, ParseException {
    super(impl);

    String base64= impl.getAkentiOpaqueCapability();

    if (base64 != null) {
      byte[] rep = Base64.decode(base64);
      ByteArrayInputStream bais = new ByteArrayInputStream(rep);
      capCert = new CapabilityCertificate(new AkentiInputStream(bais));
    }

    if (impl.getOpaqueCertificate() != null) {
      List temp = impl.getOpaqueCertificate();
      for (int i=0; i < temp.size(); i++) {
        OpaqueCertificateType x = (OpaqueCertificateType) temp.get(i);
        this.certs.add(OpaqueCertificate.fromJAXBImpl(x));
      }
      this.certs = Collections.unmodifiableList(temp);
    }
    
    if (impl.getPolicyInfo() != null) {
      List temp = impl.getPolicyInfo();
      for (int i=0; i < temp.size(); i++) {
        PolicyInfoType x = (PolicyInfoType) temp.get(i);
        this.polInfos.add(new PolicyInfo(x));
      }
      this.polInfos = Collections.unmodifiableList(temp);
    }
  }
  
 /**
   * Produces a CheckAccessResponseType to encapsulate the data in this object.
   * @throws JAXBException if mandatory data is not present
   * @return the CheckAccessResponseType
   */
  protected AkentiResponseType doToJAXBImpl() throws JAXBException {
    try {
      CheckAccessResponseType impl = Util.getObjectFactory().createCheckAccessResponseGenerated();

      if (hasCapabilityCertificate()) {
        CapabilityCertificate cert = getCapabilityCertificate();
        ByteArrayOutputStream baos = new ByteArrayOutputStream();

        cert.writeObject(new AkentiOutputStream(baos));

        String base64 = new String(Base64.encode(baos.toByteArray()));
        impl.setAkentiOpaqueCapability(base64);
      }

      if (certs != null) {
        for (int i=0; i < certs.size(); i++) {
          AkentiCertificate x = (AkentiCertificate) certs.get(i);
          impl.getOpaqueCertificate().add(OpaqueCertificate.toJAXBImpl(x));
        }
      }
      
      if (polInfos != null) {
        for (int i=0; i < polInfos.size(); i++) {
          PolicyInfo x = (PolicyInfo) polInfos.get(i);
          impl.getPolicyInfo().add(x.toJAXBImpl());
        }
      }
      
      return impl;
    } catch(IOException ex) {
      throw new JAXBException(ex.getMessage());
    }
  }
}