package lbl.akenti.messaging;

import java.io.IOException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;
import javax.xml.bind.JAXBException;
import lbl.akenti.jaxb.AkentiResponseType;
import lbl.akenti.jaxb.CheckPolicyResponseType;
import lbl.akenti.jaxb.PolicyInfoType;

/**
 * Returns all the certificates needed to authorize a Capability
 * Certificate.
 * @author Tommaso Sciortino
 */
final public
class CheckPolicyResponse extends AkentiResponse {
  private List polInfosList;

 /**
   * Constructs this <code>CheckPolicyResponse</code> with no
   * <code>PolicyInfo</code>s.
   * @param uid the id
   * @param mesg the AkentiMessages
   */  
  public CheckPolicyResponse(int uid, AkentiMessage mesg) {
    this(uid, mesg, new PolicyInfo[0]);
  }

 /**
   * Constructs this <code>CheckPolicyResponse</code> with a <code>PolicyInfo</code>.
   * @param uid the id
   * @param mesg the AkentiMessage
   * @param pr the PolicyInfo
   */
  public CheckPolicyResponse(int uid, AkentiMessage mesg, PolicyInfo pr) {
    this(uid, mesg, new PolicyInfo[] { pr });
  }

 /**
   * Constructs this <code>CheckPolicyResponse</code> with <code>PolicyInfo</code>s.
   * @param uid the id
   * @param mesg the Akentimessage
   * @param prs the PolicyInfos
   */
  public CheckPolicyResponse(int uid, AkentiMessage mesg, PolicyInfo[] prs) {
    super(uid, mesg);
    polInfosList = new ArrayList(prs.length);

    for (int i = 0; i < prs.length; ++i) {
      polInfosList.add(prs[i]);
    } 

    polInfosList = Collections.unmodifiableList(polInfosList); 
  }

 /**
   * Returns the PolicyInfos as an array.
   * @return the PolicyInfos
   */
  public PolicyInfo[] toPolicyInfosArray() {
    PolicyInfo[] ret = new PolicyInfo[polInfosList.size()];
	
    polInfosList.toArray(ret);
    return ret;
  }

 /**
   * Returns an iterator to the PolicyInfos
   * @return the iterator
   */
  public Iterator getPolicyInfos() {
    return polInfosList.listIterator();
  }
  
 /**
   * Constructs this <code>CheckPolicyRequest</code> with a
   * <code>CheckPolicyRequestType</code>.
   * @param impl the CheckPolicyRequestType
   * @throws IOException if the x509 certificate is not encoded correctly
   * @throws ParseException if mandatory data is not present
   */
  CheckPolicyResponse(CheckPolicyResponseType impl) 
    throws IOException, ParseException {
    super(impl);

    List tempList = impl.getPolicyInfo();
    Iterator iter = tempList.listIterator();

    polInfosList = new ArrayList(tempList.size());

    while (iter.hasNext()) {
      PolicyInfoType prt = (PolicyInfoType) iter.next();

      polInfosList.add(new PolicyInfo(prt));
    }

    polInfosList = Collections.unmodifiableList(polInfosList); 
  }

 /**
   * Produces a CheckPolicyResponseType to encapsulate the data in this object.
   * @throws JAXBException if mandatory data is not present
   * @return the CheckPolicyResponseType
   */
  protected AkentiResponseType doToJAXBImpl() throws JAXBException {
    CheckPolicyResponseType impl = Util.getObjectFactory().createCheckPolicyResponseGenerated();
    Iterator iter = getPolicyInfos();
    List tempList = impl.getPolicyInfo();

    while (iter.hasNext()) {
      PolicyInfo ar = (PolicyInfo) iter.next();

      tempList.add(ar.toJAXBImpl());
    } 

    return impl;
  }
}
