package lbl.akenti.parsers;

import java.text.ParseException;
import lbl.akenti.util.ParsingException;

/**
  *
  */
/**
  * Parses an expression using the grammar defined below.
  *
  * Grammar:<BR>
  *
  * <P>
  * EXPR      ->  AEXPR   { || AEXPR }
  * AEXPR     ->  NEXPER  { && NEXPR }
  * NEXPER    ->  { ! NEXPR } | PEXPER 
  * PEXPER    ->  TEXPER AOP TEXPER | ( EXPER ) 
  * TEXPER    ->  { string }
  * AOP       ->  "=" | "!=" | "<" | "<=" | ">" | ">="
  * </P>
  *
  */
final class 
ExpressionParser implements TokenType {

  private ExpressionScanner scanner;

  /**
    */
  ExpressionParser() {
  }

  /**
    */
  TreeNode parseExpression(String expr) throws ParseException {
    if (expr.equals("") || expr.equals("()")) {
      return new TreeNode(ENULL, "");
    }

    scanner = new ExpressionScanner(expr);
    scanner.advance();
    TreeNode n = orExpression();

    if (TokenMap.tokenType(scanner.getCurrentToken()) != INVALID) {
      throw new ParsingException("Did not expect a right paren");
    }
    
    return n;
  }

  /** 
    */
  private TreeNode orExpression() throws ParseException {
    TreeNode n = andExpression();
    String token = scanner.getCurrentToken();
 
    if (TokenMap.tokenType(token) == OR) {
      scanner.advance();
      n = new TreeNode(OR, OR_STRING, n, orExpression());
    }
 
    return n;
  }
 
  /**
    */
  private TreeNode andExpression() throws ParseException {
    TreeNode n = notExpression();
    String token = scanner.getCurrentToken();
 
    if (TokenMap.tokenType(token) == AND) {
      scanner.advance();
      n = new TreeNode(AND, AND_STRING, n, andExpression());
    }
 
    return n;
  }
 
  /** 
    */
  private TreeNode notExpression() throws ParseException {
    String token = scanner.getCurrentToken();

    if (TokenMap.tokenType(token) == NOT) {
      scanner.advance();
      return new TreeNode(NOT, NOT_STRING, notExpression(), null);
    } else {
      return parenExpression();
    }
  }
 
  /**
    */
  private TreeNode parenExpression() throws ParseException {
    TreeNode n;
    String token = scanner.getCurrentToken();

    switch(TokenMap.tokenType(token)) {
      case LPAREN:
        scanner.advance();
        n = orExpression();
        token = scanner.getCurrentToken();

        if (TokenMap.tokenType(token) != RPAREN) {
          throw new ParsingException("Expected right paren");
        }

        break;
      case CONST:
        String attr = scanner.getCurrentToken();
        String op = scanner.advance();
        int opType = TokenMap.tokenType(op);
         
        if (opType < GT || opType > NEQ) {
          throw new ParsingException("Expected equality testing operator after" + attr);
        }

        String value = scanner.advance();

        if (TokenMap.tokenType(value) != CONST) {
          throw new ParsingException("Expected a value for " + attr);
        }

        n = new TreeNode(opType,
                         op,
                         new TreeNode(CONST, attr), 
                         new TreeNode(CONST, value));
        break;
      default:
        throw new ParsingException("Expected a left paren or an attribute");
    }
    
    scanner.advance();
    return n;
  }
}
