package lbl.akenti.policyEngine.impl;
 
import java.io.File;
import java.io.IOException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.StringTokenizer;
import lbl.akenti.certs.AkentiCertificateFactory;
import lbl.akenti.certs.Certificate;
import lbl.akenti.certs.PolicyCertificate;
import lbl.akenti.certs.elements.AkentiIssuer;
import lbl.akenti.certs.elements.AkentiPrincipal;
import lbl.akenti.certs.elements.UseCondIssuerGroup;
import lbl.akenti.policyEngine.PolicyCollector;
import lbl.akenti.policyEngine.Runtime;
import lbl.akenti.util.AkentiConstants;
import lbl.akenti.util.UTCTime;

/**
  * @author Abdelilah Essiari
  **/
class AkentiPolicyCollector implements AkentiConstants, PolicyCollector {
  private Runtime runtime;

  AkentiPolicyCollector(Runtime runtime) {
    this.runtime = runtime;
  }

  /**
    * Loads and verifies policy certificates.
    */
  public boolean collect() {
    AkentiPolicyContext ctx = (AkentiPolicyContext) runtime.getPolicyContext();
    String resource = ctx.getResource();
    Iterator iter = parseResource(resource);

    if (iter == null) {
      ctx.setMessage(401, "Parsing resource name failed");
      return false;
    }

    resource = (String) iter.next();

    String base = RootResources.getRootResourcePath(resource);
 
    if (base == null) {
      ctx.setMessage(402, "root path unspecified");
      return false;
    }

    String policyFilePath = constructResourcePath(base, resource);
    File file = new File(policyFilePath);
 
    if (!file.isAbsolute()) {
      ctx.setMessage(403, "root path is not absolute");
      return false;
    } else if (!file.exists()) {
      ctx.setMessage(404, "root policy file does not exist");
      return false;
    } 

    PolicyCertificate policyCert = loadPolicyCertificate(policyFilePath);

    if (policyCert == null) {
      ctx.setMessage(405, "error reading root policy from " + policyFilePath);
      return false;
    } else if (!policyCert.getResource().equals(resource)) {
      ctx.setMessage(406, "wrong root resource name ");
      return false;
    } else if (!policyCert.containsCertificateAuthorityInfos()) {
      ctx.setMessage(407, "missing root cas in root policy");
      return false;
    }

    ctx.addResourcePolicy(new AkentiResourcePolicy(policyCert));

    while (iter.hasNext()) {
      resource = (String) iter.next();
      file = new File(base, resource);

      if (file.isDirectory()) {
        policyFilePath = constructResourcePath(base, resource);
        file = new File(policyFilePath);
      
        if (file.exists()) {
          policyCert = loadPolicyCertificate(policyFilePath);

          if (policyCert == null) {
            ctx.setMessage(430, "unable to read policy " + resource);
            return false;
          } else if (!policyCert.getResource().equals(resource)) {
            ctx.setMessage(431, "unable to read policy " + resource);
            return false;
          }

          ctx.addResourcePolicy(new AkentiResourcePolicy(policyCert));
        }
      } else if (file.isFile()) {
        ctx.setMessage(432, "bad resource tree");
        return false;
      }
    }

    ctx.setMessage(101, "Collect policy certificates ok");
    return true;
  }

  public boolean verify() {
    AkentiPolicyContext ctx = (AkentiPolicyContext) runtime.getPolicyContext();
    Iterator iter = ctx.getResourcePolicies();
    int policyLevel = 1;
    UTCTime now = new UTCTime();

    do {
      AkentiResourcePolicy resourcePolicy = (AkentiResourcePolicy) iter.next();
      PolicyCertificate cert = resourcePolicy.getPolicyCertificate();
 
      if (!cert.checkValidity(now)) {
        ctx.setMessage(450, "policy expired:" + resourcePolicy.getResource());
        return false;
      }

      AkentiIssuer issuer = cert.getIssuer();
      AkentiPrincipal ap = issuer.getIssuer();
      UseCondIssuerGroup[] groups = cert.getUseCondIssuerGroups();
      boolean nameVerified;
      int i = 0;

      do {
        nameVerified = groups[i++].containsUCIssuer(ap);
      } while (!nameVerified && i < groups.length);
 
      if (!nameVerified) {
        ctx.setMessage(450, "policy has bad issuer:" + resourcePolicy.getResource());
        return false;
      }

      AkentiCertificateHolder holder = new AkentiCertificateHolder(cert);

      runtime.setPolicyLevel(policyLevel++);

      if (!runtime.getTrustVerifier().verify(holder)) {
        ctx.setMessage(450, "policy not trusted:" + resourcePolicy.getResource());
        return false;
      }
    } while (iter.hasNext());

    ctx.setMessage(102, "verifying policies ok");
    return true;
  } 

  private String constructResourcePath(String base, String resource) {
    String policyFileName = System.getProperty("PolicyFileName", PolicyFileName);
    StringBuffer sb = new StringBuffer(base);

    sb.append(File.separator);
    sb.append(resource);
    sb.append(File.separator);
    sb.append(policyFileName);
    return sb.toString();
  }

  private Iterator 
  parseResource(String resource) {
    StringTokenizer st = new StringTokenizer(resource, "/ ", true);
    StringBuffer sb = new StringBuffer(resource.length()); 
    ArrayList resources = new ArrayList(st.countTokens() / 2);
 
    while (st.hasMoreTokens()) {
      String temp = st.nextToken();
 
      if (temp.equals("/") || temp.equals(" ")) {
        return null;
      }
 
      sb.append(temp);
      resources.add(sb.toString());
 
      if (!st.hasMoreTokens()) {
        return resources.iterator();
      }

      if (st.nextToken().equals(" ")) {
        return null;
      }
 
      sb.append("/");
    }
    
    return null;
  }

  private PolicyCertificate
  loadPolicyCertificate(String fileName) {
    try {
      String encType = System.getProperty("AK_CERT_ENC_TYPE", AK_CERT_ENC_TYPE);
      Certificate cert = AkentiCertificateFactory.initCertFromFile(encType, fileName);
       
      if (cert instanceof PolicyCertificate) {
        return (PolicyCertificate) cert;
      }
    } catch(IOException ioe) {
      ioe.printStackTrace();
    } catch(ParseException pe) {
      pe.printStackTrace();
    }
    
    return null;
  }
}
