package lbl.akenti.policyEngine.impl;

import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.TreeSet;
import lbl.akenti.certs.AttributeCertificate;
import lbl.akenti.certs.CapabilityCertificate;
import lbl.akenti.certs.Certificate;
import lbl.akenti.certs.PolicyCertificate;
import lbl.akenti.certs.elements.AkentiPrincipal;
import lbl.akenti.certs.elements.ConditionalActions;
import lbl.akenti.policyEngine.PolicyContext;
import lbl.akenti.policyEngine.ResourcePolicy;
import lbl.akenti.util.AkentiMessage;

/**
  * Holds information collected during CheckAccess/CollectPolicy requests.<BR>
  * 
  * @author Abdelilah Essiari
  */
public class AkentiPolicyContext implements PolicyContext {

  /**
    * Name of the resource handed to the engine. 
    */
  private String resource;

  /**
    * Name of the resource handed to the engine. 
    */
  private String resAltName;

  /**
    * Subject accessing the resource.
    */
  private AkentiPrincipal subject;

  /**
    * Subject's <code>X509</code> certificat.
    */
  private X509Certificate subjectCert;

  /**
    * Subject's <code>Capability</code> certificate.
    */
  private CapabilityCertificate capCert;

  /**
    * List of <code>Policy</code> and <code>UseCondition</code> certificates collected.
    */
  private List policies = new ArrayList();

  /**
    * List of <code>Attribute</code> certificates collected.
    */
  private List attrCerts = new ArrayList();

  /**
    * List of policies collected.
    */
  private List x509Certs = new ArrayList();

  /**
    * List of certs collected by the external evaluator.
    */
  private List extCerts = new ArrayList();

  /**
    */
  private Set desiredActions  = new TreeSet();

  /**
    */
  private List grantedActions  = new ArrayList();

  /**
    */
  private List condActionsList  = new ArrayList();

  /**
    */
  private AkentiMessage mesg = new AkentiMessage();

  /**
    * Constructs a policy context. 
    */
  AkentiPolicyContext() {
  }

  /**
    * Sets the name of the resource.
    */
  public void setResource(String resource) {
    this.resource = resource;
  }

  /**
    * Returns the name of the resource.
    */
  public String getResource()  {
    return resource;
  }

  /**
    * Sets the name of the resource.It could be different
    * the <code>resource</code> name exposed to the outside.
    */
  public void setResourceAltName(String resAltName) {
    this.resAltName = resAltName;
  }

  /**
    * Returns the resource alternative name.
    */
  public String getResourceAltName()  {
    return resAltName;
  }

  /**
    * Sets the <code>AkentiPrincipal</code> that is accessing 
    * the resource.
    */
  public void setSubject(AkentiPrincipal subject) {
    this.subject = subject;
  }

  /**
    * Returns the <code>AkentiPrincipal</code> that is accessing 
    * the resource. 
    */
  public AkentiPrincipal getSubject()  {
    return subject;
  }

  /**
    * Sets the <code>X509</code> certificate of the entity that is accessing 
    * the resource.
    */
  public void setSubjectCertificate(X509Certificate subjectCert) {
    this.subjectCert = subjectCert;
  }

  /**
    * Returns the <code>X509</code> certificate of the entity that is accessing 
    * the resource.
    */
  public X509Certificate getSubjectCertificate() {
    return subjectCert;
  }

  /**
    * Sets the <code>Capability</code> certificate of the entity that is accessing 
    * the resource.
    */
  public void setCapabilityCertificate(CapabilityCertificate capCert) {
    this.capCert = capCert;
  }

  /**
    * Returns the <code>Capability</code> certificate of the entity that is accessing 
    * the resource.
    */
  public CapabilityCertificate getCapabilityCertificate()  {
    return capCert;
  }

  /**
    */
  public ResourcePolicy[] toResourcePolicies() {
    return (ResourcePolicy[]) policies.toArray(new ResourcePolicy[0]);
  }

  /**
    * Returns an iterator over the <code>ResourcePolicies</code>
    *
    * @see ResourcePolicy
    */
  public Iterator getResourcePolicies()  {
    return policies.iterator();
  }

  /**
    */
  public ResourcePolicy getResourcePolicy(int index)  {
    return (ResourcePolicy) policies.get(index);
  }

  /**
    * Adds a <code>ResourcePolicy</code>.
    *
    * @see ResourcePolicy
    */
  public void addResourcePolicy(ResourcePolicy policy) {
    policies.add(policy);
  }

  /**
    * Adds an <code>Akenti</code> certificate.
    *
    * @see java.security.certs.X509Certificate
    */
  public void addX509Certificate(X509Certificate cert) {
    x509Certs.add(cert);
  }

  /**
    */
  public X509Certificate[] toX509Certificates() {
    return (X509Certificate[]) x509Certs.toArray(new X509Certificate[0]);
  }

  /**
    * Returns an iterator over <code>X509</code> certificates.
    *
    * @see java.security.certs.X509Certificate
    */
  public Iterator getX509Certificates() {
    return x509Certs.iterator();
  }
  
  /**
    */
  public void addAttributeCertificate(AttributeCertificate attrCert) {
    attrCerts.add(attrCert);
  }

  public AttributeCertificate[] toAttributeCertificates() {
    return (AttributeCertificate[]) attrCerts.toArray(new AttributeCertificate[0]);
  }

  /**
    * Returns an iterator over <code>Attribute</code> certificates</code>.
    */
  public Iterator getAttributeCertificates() { 
    return attrCerts.iterator();
  }

  public long getCacheInterval() {
    Iterator iter = getResourcePolicies();

    while (iter.hasNext()) {
      ResourcePolicy po = (ResourcePolicy) iter.next();
      PolicyCertificate cert = po.getPolicyCertificate();
      long cacheTime = cert.getCacheTime();
      
      if (cacheTime >= 0) {
        return cert.getCacheTime();
      }
    }

    return 0;
  }

  public Certificate[] toExternalCertificates() {
    return (Certificate[]) extCerts.toArray(new Certificate[0]);
  }

  void setMessage(int code, String message) {
    mesg = new AkentiMessage(code, message);
  }

  public AkentiMessage getAkentiMessage() {
    return mesg;
  }

  public void addExternalCertificate(Certificate cert) {
    extCerts.add(cert);
  }

  /**
    */
  public String[] toDesiredActions() {
    return (String []) desiredActions.toArray(new String[0]);
  }
 
  /**
    */
  public boolean containsDesiredActions() {
    return desiredActions.size() != 0;
  }

  /**
    */
  public void setDesiredActions(Set desiredActions) {
    if (desiredActions != null) {
      this.desiredActions = desiredActions;
    }
  } 

  /**
    */
  public Set getDesiredActions() {
    return desiredActions;
  } 

  /**
    */
  public String[] toUnconditionalActions() {
    return (String []) grantedActions.toArray(new String[0]);
  }

  public void addGrantedAction(String action) {
    grantedActions.add(action);
  }

  public void addGrantedActions(Set actions) {
    grantedActions.addAll(actions);
  }

  /**
    */
  public ConditionalActions[] toConditionalActions() {
    return (ConditionalActions[]) condActionsList.toArray(new ConditionalActions[0]);
  }

  public void addConditionalActions(ConditionalActions condActions) {
    condActionsList.add(condActions);
  }
  
}
