package lbl.akenti.policyEngine.impl;

import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.security.GeneralSecurityException;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import javax.security.auth.x500.X500Principal;
import lbl.akenti.certs.Certificate;
import lbl.akenti.certs.PolicyCertificate;
import lbl.akenti.certs.elements.AkentiIssuer;
import lbl.akenti.certs.elements.AkentiPrincipal;
import lbl.akenti.certs.elements.CertificateAuthorityInfo;
import lbl.akenti.certs.elements.Directory;
import lbl.akenti.certs.elements.DistinguishedName;
import lbl.akenti.dirServices.CertificateRetrievalFactory;
import lbl.akenti.policyEngine.CertificateCache;
import lbl.akenti.policyEngine.CertificateHolder;
import lbl.akenti.policyEngine.ResourcePolicy;
import lbl.akenti.policyEngine.Runtime;
import lbl.akenti.policyEngine.TrustVerifier;
import lbl.akenti.util.UTCTime;
import lbl.akenti.util.Util;

// NOTES MISSING VERIFICATION OF SIGNATURE
// CAS and the null issuer
// UTCTIME VALIDATION

/**
  * TrustVerifier is responsible for deciding whether certificates/subject
  * should be trusted or not. 
  *
  * @author Abdelilah Essiari
  **/
public final class AkentiTrustVerifier implements TrustVerifier {
  private Map listedCAS = new HashMap();
  private Map listedDirs = new HashMap();
  private Runtime runtime;

  /**
    */
  AkentiTrustVerifier(Runtime runtime) {
    this.runtime = runtime;
  }

  /**
    */
  public void loadTrustMaterial() {
    AkentiPolicyContext ctx = (AkentiPolicyContext) runtime.getPolicyContext(); 
    Iterator iter = ctx.getResourcePolicies();
    int caLevel = 1;

    while (iter.hasNext()) {
      ResourcePolicy resourcePolicy = (ResourcePolicy) iter.next();
      PolicyCertificate policyCert = resourcePolicy.getPolicyCertificate();
      CertificateAuthorityInfo[] infos = policyCert.getCertificateAuthorityInfos();
      Map cas = new HashMap();

      for (int i = 0; i < infos.length; ++i) {
        X509Certificate[] chain = infos[i].getChain();

        cas.put(infos[i].getDistinguishedName(), 
                new CACertHolderHolder(new AkentiX509Certificate(chain[0]), 
                                 caLevel == 1? 1 : 0,    // ONLY ROOT ARE TRUSTED 
                                 caLevel, 
                                 infos[i].getIdentityDirectories()));  

        Map previous = (Map) listedCAS.get(new Integer(caLevel - 1));

        for (int j = 1; caLevel > 1 && j < chain.length; ++j) {
          AkentiX509Certificate cert = new AkentiX509Certificate(chain[j]);
          DistinguishedName dn = cert.getSubject();

          if (previous.get(dn) == null) {  // DO NOT OVERWRITE 
            previous.put(dn, new CACertHolderHolder(cert, 0, caLevel - 1, new Directory[0]));
          }
        }
      }

      listedCAS.put(new Integer(caLevel++), cas);
    }
  }

  public CertificateHolder verifySubject(AkentiPrincipal ap) {
    AkentiCertificateHolder h = findX509Certificate(ap.getName(),
                                                    ap.getGuarantor(),
                                                    new Directory[0],
                                                    runtime.getPolicyLevel(),
                                                    null); 

    if (h == null || !verify(h)) {
      return null;
    } 

    return h;
  }

  public CertificateHolder verify(Certificate cert) {
    AkentiCertificateHolder h = new AkentiCertificateHolder(cert);
    
    verify(h);
    return h;
  }

  public boolean verify(CertificateHolder h) {
    Certificate cert = h.getCertificate();
    AkentiIssuer issuer = cert.getIssuer();
    int startLevel = getCALevelStartingAt(runtime.getPolicyLevel());
    int caLevel = startLevel;

    ((AkentiCertificateHolder) h).setTrustLevel(runtime.getPolicyLevel());

    AkentiCertificateHolder holder = findX509Certificate(issuer.getName(),
                                                         issuer.getGuarantor(),
                                                         issuer.getIdentityDirs(),
                                                         caLevel,
                                                         cert); 

    if (holder == null) {
      return false;
    } else if (holder.getTrustLevel() < 0) {
      return false;
    } else if (holder.getTrustLevel() >= caLevel) {
      return true;
    }

    ArrayList chain = new ArrayList(10);
    int depth = 10;
    
    do {
      chain.add(holder);
      cert = holder.getCertificate();
      issuer = cert.getIssuer();
      
      if (holder instanceof CACertHolderHolder) {
        caLevel = ((CACertHolderHolder) holder).caLevel;
      }

      holder = findX509Certificate(issuer.getName(),
                                   null,
                                   issuer.getIdentityDirs(),
                                   caLevel,
                                   cert); 
        
    } while ((holder != null)
          && (holder.getTrustLevel() > 0)
          && (holder.getTrustLevel() < caLevel)
          && (--depth > 0));

    if (depth == 0) {
      return false;
    } else if (holder == null) {
      Iterator iter = chain.iterator();
      
      while (iter.hasNext()) {
        holder = (AkentiCertificateHolder) iter.next();
        holder.setTrustLevel(-1);
      }
      
      return false;
    }

    Iterator iter = chain.iterator();
    
    caLevel = startLevel;

    while (iter.hasNext()) {
      holder = (AkentiCertificateHolder) iter.next();
      
      if (holder instanceof CACertHolderHolder) {
        caLevel = ((CACertHolderHolder) holder).caLevel;
      }

      holder.setTrustLevel(caLevel);
    }

    return true;
  }

  private AkentiCertificateHolder
  findX509Certificate(DistinguishedName dn,
                      DistinguishedName caDN,
                      Directory[] idDirs,
                      int startLevel,
                      Certificate cert) {
    CertificateHolder[] holders = findX509Certificates(dn, caDN, idDirs, startLevel);

    if (holders.length == 0) {
      return null;
    }

    UTCTime now = new UTCTime();

    for (int i = 0; i < holders.length; ++i) {
      Certificate signerCert = holders[i].getCertificate();
 
      if (!signerCert.checkValidity(now)) {
      } else if (cert == null) {
        return (AkentiCertificateHolder) holders[i];
      } else {
        try {
          if (cert.verify(((AkentiX509Certificate) signerCert).getPublicKey())) {
            return (AkentiCertificateHolder) holders[i];
          } else {
          }
        } catch(IOException ex) {
          ex.printStackTrace();
        } catch(GeneralSecurityException ex) {
          ex.printStackTrace();
        }
      }
    }

    return null;
  }

  private CertificateHolder[]
  findX509Certificates(DistinguishedName dn,
                       DistinguishedName caDN,
                       Directory[] idDirs,
                       int startLevel) {
    // IS A CA'S CERTIFICATE?
    CACertHolderHolder holder = (CACertHolderHolder) getCAStartingAtLevel(dn, startLevel);
       
    if (holder != null) {
      return new CertificateHolder[] {  holder };
    } 

    CertificateCache certCache = runtime.getCertificateCache();
    String key = dn.to400String();  

    CertificateHolder[] holders = certCache.getCertificateHolders(key);

    if (holders.length != 0) {
      return holders;
    }

    if (idDirs.length == 0) {
      idDirs = getIDDirsStartingAtLevel(startLevel);

      if (idDirs.length == 0) {
        return new CertificateHolder[0];
      }
    }

    X500Principal princ = new X500Principal(key);
    byte[] der;

    try {
      der = princ.getEncoded();
    } catch(Exception ex) {
      ex.printStackTrace();
      throw new RuntimeException();
    }

    Map args = new HashMap();
 
    args.put("type", "X509");
    args.put("hash", Util.createHashName(der));
    args.put("base", key);

    List holderList = new ArrayList(1);

    for (int i = 0; holderList.size() == 0 && i < idDirs.length; ++i) {
      X509Certificate[] certs = retrieveCertificates(idDirs[i], args);
      
      for (int j = 0; j < certs.length; ++j) {
        AkentiX509Certificate cert = new AkentiX509Certificate(certs[j]);

        // if (cert.getSubject().equals(dn) && (caDN == null || caDN.equals(cert.getIssuer().getName()))) {
        if (cert.getSubject().equals(dn)) {
          holderList.add(certCache.addCertificate(key, cert));
        }
      }
    }

    return (CertificateHolder[]) holderList.toArray(new CertificateHolder[0]);
  }

  private Directory[] 
  getIDDirsStartingAtLevel(int startLevel) {
    int caLevel = getCALevelStartingAt(startLevel);
    Integer key = new Integer(caLevel);
    Directory[] idDirs = (Directory []) listedDirs.get(key);
    
    if (idDirs == null) {
      Map cas = (Map) listedCAS.get(new Integer(caLevel));
      Iterator iter = cas.values().iterator();
      ArrayList temp = new ArrayList();

      while (iter.hasNext()) {
        CACertHolderHolder holder = (CACertHolderHolder) iter.next();
 
        for (int i = 0; i < holder.idDirs.length; ++i) {
          temp.add(holder.idDirs[i]);
        }
      }

      idDirs = (Directory[]) temp.toArray(new Directory[0]);
      listedDirs.put(key, idDirs);
    }

    return idDirs;
  }

  private CACertHolderHolder 
  getCAStartingAtLevel(DistinguishedName dn, int startLevel) {
    int caLevel = getCALevelStartingAt(startLevel);
    Map cas = (Map) listedCAS.get(new Integer(caLevel));

    return (CACertHolderHolder) cas.get(dn);
  }

  private int getCALevelStartingAt(int startLevel) {
    Map cas = (Map) listedCAS.get(new Integer(startLevel));

    while (cas.size() == 0) {
      cas = (Map) listedCAS.get(new Integer(--startLevel));
    }

    return startLevel;
  }

  private X509Certificate[] retrieveCertificates(Directory dir, Map args) {
    try {
      URL url = dir.parseDirectory();
 
      return CertificateRetrievalFactory.retrieveX509Certificates(url, args);
    } catch(MalformedURLException ex) {
      ex.printStackTrace();
      return new X509Certificate[0];
    }
  }

  class CACertHolderHolder extends AkentiCertificateHolder {
    int caLevel;
    Directory[] idDirs;

    CACertHolderHolder(Certificate cert, 
                 int trustLevel, 
                 int caLevel, 
                 Directory[] idDirs) {
      super(cert, trustLevel);
      this.caLevel = caLevel;
      this.idDirs = idDirs;
    }
  }
}
