package lbl.akenti.protocolHandler;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.security.cert.CertificateException;
import java.util.ArrayList;
import java.util.StringTokenizer;
import lbl.akenti.certs.AkentiCertificateFactory;
import lbl.akenti.certs.PolicyCertificate;
import lbl.akenti.messaging.AkentiMessage;
import lbl.akenti.messaging.AkentiRequest;
import lbl.akenti.messaging.AkentiResponse;
import lbl.akenti.messaging.CertificateInfo;
import lbl.akenti.messaging.CheckAccessRequest;
import lbl.akenti.messaging.CheckAccessResponse;
import lbl.akenti.messaging.CheckPolicyRequest;
import lbl.akenti.messaging.CheckPolicyResponse;
import lbl.akenti.messaging.CreateResourceRequest;
import lbl.akenti.messaging.CreateResourceResponse;
import lbl.akenti.messaging.DeletePolicyCertificateRequest;
import lbl.akenti.messaging.DeletePolicyCertificateResponse;
import lbl.akenti.messaging.DeleteResourceRequest;
import lbl.akenti.messaging.DeleteResourceResponse;
import lbl.akenti.messaging.GetResourceInfoRequest;
import lbl.akenti.messaging.GetResourceInfoResponse;
import lbl.akenti.messaging.PolicyInfo;
import lbl.akenti.messaging.ResourceBrowseRequest;
import lbl.akenti.messaging.ResourceBrowseResponse;
import lbl.akenti.messaging.UploadPolicyCertificateRequest;
import lbl.akenti.messaging.UploadPolicyCertificateResponse;
import lbl.akenti.policyEngine.check.CertInfo;
import lbl.akenti.policyEngine.check.CertInfoCollector;
import lbl.akenti.policyEngine.check.PolicyChecker;
import lbl.akenti.util.AkentiConstants;
import lbl.akenti.util.ConfFileParser;

/**
 * Processes AkentiRequests and spits out the appropriate
 * AkentiResponse. All of these will fail if the configuration
 * file has not yest been loaded.
 * @see lbl.akenti.util.ConfFileParser
 */
public class LocalProtocolHandler extends ProtocolHandler implements AkentiConstants {
  private final static String ps = System.getProperty("file.separator");
  
  /**
   * Checks a users access to a resource.
   * @param req the request
   * @throws IOException not thrown
   * @return the response
   */
  public CheckAccessResponse
  process(CheckAccessRequest req) throws IOException {
    return null;
  }
  
  /**
   * Checks a resource and returns all associated certificates.
   * @param req the request
   * @throws IOException not thrown
   * @return the response
   */
  public CheckPolicyResponse
  process(CheckPolicyRequest req) throws IOException {
    
    PolicyChecker policyChecker = new PolicyChecker();
    
    CertInfoCollector ciCol = policyChecker.checkPolicy(req.getResource());
    boolean worked = ciCol.isStatus();
    
    AkentiMessage mesg = worked? new AkentiMessage(150, "Policy is OK") : new AkentiMessage(450,"failed verifying policies");
    
    CertInfo[] cinfos = ciCol.getCertInfos();
    
    if (cinfos.length == 0) {
      return new CheckPolicyResponse(req.getID(),mesg);
    }
    
    ArrayList pinfos = new ArrayList();
    
    for (int i = 0; i < cinfos.length; ++i) {
      CertInfo certInfo = cinfos[i];
      String type = certInfo.getType();
      
      if (type.equals("dummy")) { // root level error
        
        CertificateInfo info = new CertificateInfo(certInfo.getAkentiCertificate(),certInfo.getAkentiMessages(),certInfo.getLocation(),certInfo.isUsed());
        
        PolicyInfo pinfo = new PolicyInfo(info,certInfo.getResource(),!certInfo.isError());
        
        pinfos.add(pinfo);
        
        if (certInfo.getAkentiMessages().length == 1) { // should always be 1 here
          AkentiMessage[] mesgs = certInfo.getAkentiMessages();
          return new CheckPolicyResponse(req.getID(),mesgs[0]);
        }
        
      } else if (type.equals("Policy")) { // found a decent policy cert
        ArrayList info = new ArrayList();
        info.add(new CertificateInfo(certInfo.getAkentiCertificate(),certInfo.getAkentiMessages(),certInfo.getLocation(),certInfo.isUsed()));
        
        for (int j = 0; j < cinfos.length; ++j) {
          CertInfo ucCertInfo = cinfos[j];
          String ucType = ucCertInfo.getType();
          if (ucType.equals("UseCondition") && ucCertInfo.getResource().equals(certInfo.getResource())) {
            info.add(new CertificateInfo(ucCertInfo.getAkentiCertificate(),ucCertInfo.getAkentiMessages(),ucCertInfo.getLocation(),ucCertInfo.isUsed()));
            break;
          }
          
        }
        
        CertificateInfo[] certs = (CertificateInfo[]) info.toArray(new CertificateInfo[info.size()]);
        
        PolicyInfo pinfo = new PolicyInfo(certs,certInfo.getResource(),!certInfo.isError());
        
        pinfos.add(pinfo);
      }
    }
    
    return new CheckPolicyResponse(req.getID(),mesg,(PolicyInfo[]) pinfos.toArray(new PolicyInfo[pinfos.size()]));
    
  }
  
  /**
   * Returns the resources immediatly below the one mentioned in
   * the request.
   * @param req the request
   * @throws IOException not thrown
   * @return the response
   */
  public ResourceBrowseResponse
  process(ResourceBrowseRequest req) throws IOException {
    String resource = req.getResource();
    
    if (resource.equals("")) {
      
      String[] dirs = ConfFileParser.getResources();
      
      return new ResourceBrowseResponse(req.getID(),new AkentiMessage(162, "Resource Browse is OK, listing root resources"),dirs);
    }
    
    String[] resources = parseResource(resource);
    
    if (resource == null) {
      return new ResourceBrowseResponse(req.getID(),new AkentiMessage(701, "parsing resource name failed"));
    }
    
    String base = ConfFileParser.getResourcePath(resources[0]);
    
    if (base == null || base.equals("")) {
      return new ResourceBrowseResponse(req.getID(),new AkentiMessage(702, "cannot find RootPath in config file"));
    }
    
    File dir = new File(base,resource);
    
    if (!dir.isAbsolute()) {
      return new ResourceBrowseResponse(req.getID(),new AkentiMessage(703, "RootPath found in config is not absolute"));
    } else if (!dir.isDirectory()) {
      return new ResourceBrowseResponse(req.getID(),new AkentiMessage(704, "resource does not exist"));
    }
    
    ArrayList dirs = new ArrayList();
    
    String[] contents = dir.list();
    for (int i = 0; i < contents.length; i++) {
      File contentObject = new File(dir.getAbsolutePath(),contents[i]);
      
      if (contentObject.isDirectory()) {
        dirs.add(contentObject.getName());
      }
    }
    
    String[] d = (String[]) dirs.toArray(new String[dirs.size()]);
    
    return new ResourceBrowseResponse(req.getID(),new AkentiMessage(161, "Resource Browse is OK"),d);
  }
  
  /**
   * Creates the resource mentioned in the request.
   * @param req the request
   * @throws IOException not thrown
   * @return the response
   */
  public CreateResourceResponse
  process(CreateResourceRequest req) throws IOException {
    String resource = req.getResource();
    
    String[] resources = parseResource(resource);
    
    if (resource == null) {
      return new CreateResourceResponse(req.getID(),new AkentiMessage(711, "parsing resource name failed"));
    }
    
    String base = ConfFileParser.getResourcePath(resources[0]);
    
    if (base == null || base.equals("")) {
      return new CreateResourceResponse(req.getID(),new AkentiMessage(712, "cannot find RootPath in config file"));
    }
    
    File dir = new File(base,resource);
    
    if (!dir.isAbsolute()) {
      return new CreateResourceResponse(req.getID(),new AkentiMessage(713, "RootPath found in config is not absolute"));
    } else if (dir.exists()) {
      if (dir.isDirectory()) {
        return new CreateResourceResponse(req.getID(),new AkentiMessage(714, "resource already exists"));
      } else {
        return new CreateResourceResponse(req.getID(),new AkentiMessage(715, "resource name already in use"));
      }
    }
    
    if (resources.length == 1) {
      return new CreateResourceResponse(req.getID(),new AkentiMessage(716, "root resource cannot be created through a request"));
    }
    
    if (dir.mkdir()) {
      return new CreateResourceResponse(req.getID(),new AkentiMessage(171, "Create Resource is OK"));
    } else {
      return new CreateResourceResponse(req.getID(),new AkentiMessage(717, "resource could not be created"));
    }
  }
  
  /**
   * Deletes the resource mentioned in the request.
   * @param req the request
   * @throws IOException not thrown
   * @return the response
   */
  public DeleteResourceResponse
  process(DeleteResourceRequest req) throws IOException {
    String resource = req.getResource();
    
    String[] resources = parseResource(resource);
    
    if (resource == null) {
      return new DeleteResourceResponse(req.getID(),new AkentiMessage(751, "parsing resource name failed"));
    }
    
    String base = ConfFileParser.getResourcePath(resources[0]);
    
    if (base == null || base.equals("")) {
      return new DeleteResourceResponse(req.getID(),new AkentiMessage(752, "cannot find RootPath in config file"));
    }
    
    File dir = new File(base,resource);
    
    if (!dir.isAbsolute()) {
      return new DeleteResourceResponse(req.getID(),new AkentiMessage(753, "RootPath found in config is not absolute"));
    } else if (!dir.exists()) {
      return new DeleteResourceResponse(req.getID(),new AkentiMessage(754, "resource doesn't exists"));
    }
    
    if (resources.length == 1) {
      return new DeleteResourceResponse(req.getID(),new AkentiMessage(755, "root resource cannot be deleted through a request"));
    }
    
    if (dir.list().length != 0) {
      return new DeleteResourceResponse(req.getID(),new AkentiMessage(756, "resource could not be deleted, it is not empty"));
    }
    
    if (!dir.delete()) {
      return new DeleteResourceResponse(req.getID(),new AkentiMessage(757, "resource could not be deleted"));
    } else {
      return new DeleteResourceResponse(req.getID(),new AkentiMessage(176, "Delete Resource is OK"));
    }
  }
  
  /**
   * Uploads the policy certificate in the request.
   * @param req the request
   * @throws IOException not thrown
   * @return the response
   */
  public UploadPolicyCertificateResponse
  process(UploadPolicyCertificateRequest req) throws IOException {
    PolicyCertificate polCert = req.getCertificate();
    String resource = polCert.getResource();
    
    String[] resources = parseResource(resource);
    
    if (resource == null) {
      return new UploadPolicyCertificateResponse(req.getID(),new AkentiMessage(721, "parsing resource name failed"));
    }
    
    String base = ConfFileParser.getResourcePath(resources[0]);
    
    if (base == null || base.equals("")) {
      return new UploadPolicyCertificateResponse(req.getID(),new AkentiMessage(722, "cannot find RootPath in config file"));
    }
    
    File dir = new File(base,resource);
    
    
    if (!dir.isAbsolute()) {
      return new UploadPolicyCertificateResponse(req.getID(),new AkentiMessage(723, "RootPath found in config is not absolute"));
    } else if (!dir.isDirectory()) {
      return new UploadPolicyCertificateResponse(req.getID(),new AkentiMessage(724, "resource does not exist"));
    }
    
    String fileName = System.getProperty("PolicyFileName", PolicyFileName);
    
    File polFile = new File(dir,fileName);
    String encType = System.getProperty("AK_CERT_ENC_TYPE", AK_CERT_ENC_TYPE);
    
    try {
      AkentiCertificateFactory.writeCertToFile(encType, polCert, polFile.getAbsolutePath());
    } catch (CertificateException ex) {
      return new UploadPolicyCertificateResponse(req.getID(),new AkentiMessage(725, "certificate could not be written"));
    }
    
    return new UploadPolicyCertificateResponse(req.getID(),new AkentiMessage(181, "Upload Certificate is OK"));
    
  }
  
  /**
   * Searches for a policy certificate in the resource mentioned
   * in the request. If a policy certificiate is found it is
   * deleted. Note: this will not delete a policy certificate
   * if the resource inheirits it's policy from a higher resource.
   * @param req the request
   * @throws IOException not thrown
   * @return the response
   */
  public DeletePolicyCertificateResponse
  process(DeletePolicyCertificateRequest req) throws IOException {
    String resource = req.getResource();
    
    String[] resources = parseResource(resource);
    
    if (resource == null) {
      return new DeletePolicyCertificateResponse(req.getID(),new AkentiMessage(731, "parsing resource name failed"));
    }
    
    String base = ConfFileParser.getResourcePath(resources[0]);
    
    if (base == null || base.equals("")) {
      return new DeletePolicyCertificateResponse(req.getID(),new AkentiMessage(732, "cannot find RootPath in config file"));
    }
    
    String fileName = System.getProperty("PolicyFileName", PolicyFileName);
    
    File certFile = new File(base + ps + resource,fileName);
    
    if (!certFile.isAbsolute()) {
      return new DeletePolicyCertificateResponse(req.getID(),new AkentiMessage(733, "RootPath found in config is not absolute"));
    } else if (!certFile.isFile()) {
      return new DeletePolicyCertificateResponse(req.getID(),new AkentiMessage(734, "resource policy certificate does not exist"));
    }
    
    certFile.delete();
    return new DeletePolicyCertificateResponse(req.getID(),new AkentiMessage(191, "Delete Certificate is OK"));
  }
  
  /**
   * Searches for the hint file for the resource named in the
   * request and returns it if it exists.
   * @param req the request
   * @throws IOException not thrown
   * @return the response
   */
  public GetResourceInfoResponse
  process(GetResourceInfoRequest req) throws IOException {
    String resource = req.getResource();
    
    String[] resources = parseResource(resource);
    
    if (resource == null) {
      return new GetResourceInfoResponse(req.getID(),new AkentiMessage(741, "parsing resource name failed"));
    }
    
    String base = ConfFileParser.getResourcePath(resources[0]);
    
    if (base == null || base.equals("")) {
      return new GetResourceInfoResponse(req.getID(),new AkentiMessage(742, "cannot find RootPath in config file"));
    }
    
    File dir = new File(base,resource);
    
    if (!dir.isAbsolute()) {
      return new GetResourceInfoResponse(req.getID(),new AkentiMessage(743, "RootPath found in config is not absolute"));
    } else if (!dir.isDirectory()) {
      return new GetResourceInfoResponse(req.getID(),new AkentiMessage(744, "resource does not exist"));
    }
    
    String hintFileName = null;
    String hintName = System.getProperty("ResAttrFileName", ResAttrFileName);
    
    for (int i=resources.length-1; i >= 0; i--) {
      File hintFile = new File(base + ps + resources[i],hintName);
      if (hintFile.exists() && hintFile.isFile()) {
        hintFileName = hintFile.getAbsolutePath();
        break;
      }
    }
    
    if (hintFileName == null) {
      return new GetResourceInfoResponse(req.getID(),new AkentiMessage(746, "no hints file found"));
    }

    StringBuffer content = new StringBuffer();
    
    BufferedReader in = new BufferedReader(new FileReader(hintFileName));
    
    try {
      String c = null;
      while ((c = in.readLine()) != null) {
        content.append(c + "\n");
      }
    } catch (IOException ex) {
      return new GetResourceInfoResponse(req.getID(),new AkentiMessage(745, "there were errors reading the hints file"));
    }
    
    in.close();

    AkentiMessage mesg = new AkentiMessage(196, "Get Resource Info is OK");
    
    return new GetResourceInfoResponse(req.getID(), mesg, content.toString());
  }
  
  /**
   * Processes a single request.
   * @param req the request
   * @throws IOException never thrown
   * @return the response
   */
  public AkentiResponse
  process(AkentiRequest req) throws IOException {
    if (req instanceof CheckAccessRequest) {
      return process((CheckAccessRequest) req);
    } else if (req instanceof CheckPolicyRequest) {
      return process((CheckPolicyRequest) req);
    } else if (req instanceof ResourceBrowseRequest) {
      return process((ResourceBrowseRequest) req);
    } else if (req instanceof CreateResourceRequest) {
      return process((CreateResourceRequest) req);
    } else if (req instanceof DeleteResourceRequest) {
      return process((DeleteResourceRequest) req);
    } else if (req instanceof UploadPolicyCertificateRequest) {
      return process((UploadPolicyCertificateRequest) req);
    } else if (req instanceof DeletePolicyCertificateRequest) {
      return process((DeletePolicyCertificateRequest) req);
    } else if (req instanceof DeletePolicyCertificateRequest) {
      return process((DeletePolicyCertificateRequest) req);
    } else if (req instanceof GetResourceInfoRequest) {
      return process((GetResourceInfoRequest) req);
    } else {
      throw new IOException("Unrecognized Akenti Request type");
    }
  }
  
  /**
   * Processes multiple requests.
   * @param requests the requests
   * @throws IOException never thrown
   * @return the responses
   */
  public AkentiResponse[]
  process(AkentiRequest[] requests) throws IOException {
    AkentiResponse[] responses = new AkentiResponse[requests.length];
    for (int i=0; i < requests.length; i++) {
      responses[i] = process(requests[i]);
    }
    return responses;
  }
  
  private static String[] parseResource(String resource) {
    StringTokenizer st = new StringTokenizer(resource, "/ ", true);
    boolean ret = false;
    StringBuffer name = new StringBuffer();
    
    ArrayList resourcesBuffer = new ArrayList();
    
    while (st.hasMoreTokens()) {
      String temp = st.nextToken();
      
      if (temp.equals("/") || temp.equals(" ")) {
        break;
      }
      
      name.append(temp);
      resourcesBuffer.add(name.toString());
      ret = true;
      
      if (!st.hasMoreTokens()) {
        break;
      }
      
      if (st.nextToken().equals(" ")) {
        ret = false;
        break;
      }
      
      name.append("/");
      ret = false;
    }
    
    if (!ret) {
      return null;
    } else {
      return (String[]) resourcesBuffer.toArray(new String[resourcesBuffer.size()]);
    }
  }
  
}