package lbl.akenti.protocolHandler;

import java.io.IOException;
import java.net.URL;
import java.util.Vector;
import lbl.akenti.messaging.AkentiRequest;
import lbl.akenti.messaging.AkentiResponse;
import lbl.akenti.messaging.AkentiResponses;
import lbl.akenti.messaging.CheckAccessRequest;
import lbl.akenti.messaging.CheckAccessResponse;
import lbl.akenti.messaging.CheckPolicyRequest;
import lbl.akenti.messaging.CheckPolicyResponse;
import lbl.akenti.messaging.CreateResourceRequest;
import lbl.akenti.messaging.CreateResourceResponse;
import lbl.akenti.messaging.DeletePolicyCertificateRequest;
import lbl.akenti.messaging.DeletePolicyCertificateResponse;
import lbl.akenti.messaging.DeleteResourceRequest;
import lbl.akenti.messaging.DeleteResourceResponse;
import lbl.akenti.messaging.GetResourceInfoRequest;
import lbl.akenti.messaging.GetResourceInfoResponse;
import lbl.akenti.messaging.ResourceBrowseRequest;
import lbl.akenti.messaging.ResourceBrowseResponse;
import lbl.akenti.messaging.UploadPolicyCertificateRequest;
import lbl.akenti.messaging.UploadPolicyCertificateResponse;
import org.apache.soap.Constants;
import org.apache.soap.SOAPException;
import org.apache.soap.encoding.SOAPMappingRegistry;
import org.apache.soap.rpc.Call;
import org.apache.soap.rpc.Parameter;
import org.apache.soap.rpc.Response;
import org.apache.soap.util.xml.QName;

/**
 * Wraps AkentiRequests and sends them to an <i>Akenti Server</i> to retrieve
 * the appropriate AkentiResponse.
 */
public class RemoteProtocolHandler extends ProtocolHandler {
  
  private URL url;
  private static SOAPMappingRegistry smr;
  
  /**
   * Creates a <code>RemoteProtocolHandler</code> pointing to
   * an AkentiServer
   * @param url the url of the AkentiServer
   */
  public RemoteProtocolHandler(URL url) {
    this.url = url;
  }
  
  /**
   * Checks a users access to a resource.
   * @param req the request
   * @throws IOException thrown if a connection cannot be made
   * @return the response
   */
  public CheckAccessResponse
  process(CheckAccessRequest req) throws IOException {
    return (CheckAccessResponse) process((AkentiRequest) req);
  }
  
  /**
   * Checks a resource and returns all associated certificates.
   * @param req the request
   * @throws IOException not thrown
   * @return the response
   */
  public CheckPolicyResponse
  process(CheckPolicyRequest req) throws IOException {
    return (CheckPolicyResponse) process((AkentiRequest) req);
  }
  
  /**
   * Returns the resources immediatly below the one mentioned in
   * the request.
   * @param req the request
   * @throws IOException thrown if a connection cannot be made
   * @return the response
   */
  public ResourceBrowseResponse
  process(ResourceBrowseRequest req) throws IOException {
    return (ResourceBrowseResponse) process((AkentiRequest) req);
  }
  
  /**
   * Creates the resource mentioned in the request.
   * @param req the request
   * @throws IOException thrown if a connection cannot be made
   * @return the response
   */
  public CreateResourceResponse
  process(CreateResourceRequest req) throws IOException {
    return (CreateResourceResponse) process((AkentiRequest) req);
  }
  
  /**
   * Deletes the resource mentioned in the request.
   * @param req the request
   * @throws IOException thrown if a connection cannot be made
   * @return the response
   */
  public DeleteResourceResponse
  process(DeleteResourceRequest req) throws IOException {
    return (DeleteResourceResponse) process((AkentiRequest) req);
  }
  
  /**
   * Uploads the policy certificate in the request.
   * @param req the request
   * @throws IOException thrown if a connection cannot be made
   * @return the response
   */
  public UploadPolicyCertificateResponse
  process(UploadPolicyCertificateRequest req) throws IOException {
    return (UploadPolicyCertificateResponse) process((AkentiRequest) req);
  }
  
  /**
   * Searches for a policy certificate in the resource mentioned
   * in the request. If a policy certificiate is found it is
   * deleted. Note: this will not delete a policy certificate
   * if the resource inheirits it's policy from a higher resource.
   * @param req the request
   * @throws IOException thrown if a connection cannot be made
   * @return the response
   */
  public DeletePolicyCertificateResponse
  process(DeletePolicyCertificateRequest req) throws IOException {
    return (DeletePolicyCertificateResponse) process((AkentiRequest) req);
  }
  
  /**
   * Searches for the hint file for the resource named in the
   * request and returns it if it exists.
   * @param req the request
   * @throws IOException thrown if a connection cannot be made
   * @return the response
   */
  public GetResourceInfoResponse
  process(GetResourceInfoRequest req) throws IOException {
    return (GetResourceInfoResponse) process((AkentiRequest) req);
  }
  
  /**
   * Processes a single request.
   * @param req the request
   * @throws IOException thrown if a connection cannot be made
   * @return the response
   */
  public AkentiResponse
  process(AkentiRequest req) throws IOException {
    return process(new AkentiRequest[] { req })[0];
  }
  
  /**
   * Processes multiple requests.
   * @param requests the requests
   * @throws IOException thrown if a connection cannot be made
   * @return the responses
   */
  public AkentiResponse[]
  process(AkentiRequest[] requests) throws IOException {
    try {
      if (requests.length == 0) {
        throw new RuntimeException();
      }
      
      Call call = new Call();
      
      call.setTargetObjectURI("http://dsd.lbl.gov/Akenti/2004/05/AkentiProtocol");
      call.setMethodName("AkentiRequests");
      call.setSOAPMappingRegistry(getRegistry());
      call.setSOAPTransport(new SOAPHTTPConnection());
      
      Vector params = new Vector();
      
      for (int i = 0; i < requests.length; ++i) {
        Parameter param = new Parameter("", AkentiRequest.class , requests[i], Constants.NS_URI_SOAP_ENC);
        
        params.addElement(param);
      }
      
      call.setParams(params);
      
      Response resp = call.invoke(url, "");
      
      if (resp.generatedFault()) {
        throw new IOException(resp.getFault().toString());
      }
      
      Parameter result = resp.getReturnValue();
      AkentiResponses responsesList = (AkentiResponses) result.getValue();
      return responsesList.toAkentiResponsesArray();
    } catch(SOAPException ex) {
      throw new IOException(ex.toString());
    }
  }
  
  protected SOAPMappingRegistry getRegistry() {
    if (smr == null) {
      synchronized(ProtocolHandler.class) {
        if (smr == null) {
          smr = new SOAPMappingRegistry();
          
          AkentiMessagingSerializer srl = new AkentiMessagingSerializer();
          QName qn1 = new QName("", "");
          
          smr.mapTypes(Constants.NS_URI_SOAP_ENC, qn1, AkentiRequest.class, srl, null);
          
          QName qn2 = new QName("http://dsd.lbl.gov/Akenti/2004/05/AkentiProtocol", "akp:AkentiResponses");
          
          smr.mapTypes(Constants.NS_URI_SOAP_ENC, qn2, AkentiResponses.class, srl, srl);
          
        }
      }
    }
    
    return smr;
  }
  
}