package lbl.akenti.servers.AKMonitor.gui;

/**
  * LogHandler.java
  *
  * by Abdelilah Essiari
  */
import java.io.IOException;
import java.io.DataInputStream;
import java.io.BufferedInputStream;
import java.io.DataOutputStream;
import java.io.InterruptedIOException;
import java.net.Socket;
import java.net.UnknownHostException;
/**
  * LogHandler handles the reading of <code>StatusEvents</code>
  * coming from the AkentiMonitorServer.
  *
  **/
public class LogHandler implements Runnable {
  /* input/output streams */
  private DataInputStream in;
  private DataOutputStream out;
  private Thread reader; 
  
  /* to be informed of connection state events. */
  private ConnectionListener connectionListener;
  
  /* to be informed of incoming StatusEvents */
  private StatusListener statusListener;
  
  /* Contructor */
  private LogHandler() { }
  
  /* 
   * connect to a server.
   *
   * @param hostname host to connect
   * @param port     port number 
   * @param uid      user id 
   **/
  public void connect(String hostname, int port, String uid) {
    int code;

    try {
      Socket socket = new Socket(hostname, port);

      /* use time out (10 minutes) */
      socket.setSoTimeout(10 * 60 * 1000);                 
      in = new DataInputStream(new BufferedInputStream(socket.getInputStream()));
      out = new DataOutputStream(socket.getOutputStream());
      write("UID=" + uid);
      code = ConnectionEvent.CONNECTED;
      reader = new Thread(this);
      reader.start();
    } catch(UnknownHostException e) {
      code = ConnectionEvent.NO_RESPONSE;
    } catch(IOException e) {
      code = ConnectionEvent.IO_ERROR;
    } 

    connectionListener.connectionStateChanged(new ConnectionEvent(this, code));
  }
  
  /*  
   * a simple write routine.
   **/
  private void write(String str) throws IOException {
    out.writeInt((str.length()));
    out.writeBytes(str);
  }

  /*
   * used for caching.
   */
  protected byte [] buffer;

  /* 
   * a simple read routine. 
   **/
  private String read() throws IOException  {
    int len = in.readInt();

    /* allocate new buffer if we need more space */
    if (buffer == null || buffer.length < len) {
      buffer = new byte[len];
    }

    in.readFully(buffer, 0, len);
    
    return new String(buffer, 0, len);
  }
  
  /*
   * we only handle one <code>Connectionlistener</code>.
   **/
  public void addConnectionListener(ConnectionListener connectionListener) {
    this.connectionListener = connectionListener;
  }

  /*
   * we only handle one <code>StatusListener</code>.
   **/
  public void addStatusListener(StatusListener statusListener) {
    this.statusListener = statusListener;
  }

  /*
   * say bye to server, then close streams.
   * Note: the disconnect message can come from two sources:
   *           the user, or the LogHandler itself 
   *
   * @see LogHandler#run()
   * @return false if we were already disconnected.
   *         true otherwise.
   **/
  public synchronized boolean disconnect() {
    try {
      if (in != null) {
        write("HELLO SERVER I AM LEAVING");
        reader.interrupt();
        in.close();
        out.close();
        return true;
      }

      return false;
    } catch(IOException e) {
      return true;
    } finally {
      reader = null;
      buffer = null;
      in = null;
      out = null;
    }
  }

  /**
    * read events as long as no errors occur 
    * or the socket does not time out.
    * 
    **/
  public void run() {
    ConnectionEvent evt = null;
    
    try { 
      while (true) {
        String rep = read();
        int index = rep.indexOf("#");
        int id = Integer.parseInt(rep.substring(0, index));
         
        StatusEvent event = null;

        if (id > 0) { 
          event = new StatusEvent(rep.substring(index + 2)); 
        }

        statusListener.addStatusEvent(id , event);
      }
    } catch (InterruptedIOException e) {
      evt = new ConnectionEvent(this, ConnectionEvent.TIMED_OUT);
    } catch (IOException e) { 
      evt = new ConnectionEvent(this, ConnectionEvent.IO_ERROR);
    } catch(Exception e) {
      evt = new ConnectionEvent(this, ConnectionEvent.RUNTIME_ERROR, e.toString());
    } finally {
      /* 
       * here we only notify the connection listener if we are the ones who
       * actually closed the streams.
       **/
      if (disconnect()) {
        connectionListener.connectionStateChanged(evt);
      }
    } 
  }
  
  /* LogHandler */
  static LogHandler logHandler;
  
  /* 
   * instantiate the log handler if one does not already exits 
   **/
  static synchronized LogHandler getLogHandler() {
    if (logHandler == null) {
      logHandler = new LogHandler();
    }

    return logHandler;
  }
}
    

