package lbl.akenti.servers.CacheManager;

/**
  *
  * Success:
  *
  *   101 OK
  *   102 OK contents of old cert overwriten
  *   103 OK but cert key was not in internal table
  *   104 OK file was not there (Flush)
  *
  * Failure: 
  *
  *   201 writing to certfile failed
  *   202 reading certfile failed
  *   203 parsing certfile failed
  *   204 no cert found
  *   205 cert has expired
  *   206 deleting certfile failed
  *   207 deleting certs failed
  *
  * Client Errors
  *
  *   301 bad request no cert to be updated
  *   302 request not supported 
  *
  * Server Errors
  *
  *   400 directory does not exists
  *   401 unable to create directory(ies) to store cert.
  *   402 a file that should be a directory
  *   403 a directory that should be a file
  *   404 certfile expected but is not there
  *   405 no write access to certfile
  *   406 no read access to certfile
  *   407 cannot write to directory
  */

import java.net.Socket;
import java.io.File;
import java.io.LineNumberReader;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.io.IOException;
import java.net.SocketException;
import java.io.InterruptedIOException;
import java.io.FileNotFoundException;
import java.text.ParseException;

public class CacheHandler implements Runnable {

  private Socket sock;

  private File baseDir;

  /**
    */
  public CacheHandler(Socket sock, File baseDir) {
    if (sock == null || !baseDir.isDirectory() || !baseDir.canWrite()) {
      throw new IllegalArgumentException();
    }

    this.sock = sock;
    this.baseDir = baseDir;
  }
    
  /**
    */
  public void run() {

    try {
      sock.setSoTimeout(1 * 60  * 1000);

      while (true) {
        CacheRequest req = new CacheRequest();


        req.readExternal(
          new LineNumberReader(new InputStreamReader(sock.getInputStream()))
        );

        System.err.println(req);

        CacheResponse resp = new CacheResponse(req); 

        service(req, resp);
        resp.writeExternal(new PrintWriter(sock.getOutputStream()));
        System.err.println(resp);
      }
    } catch(SocketException ex) {
      System.err.println(ex);
    } catch(ParseException ex) {
      System.err.println(ex);
    } catch(InterruptedIOException ex) {
      System.err.println(ex);
    } catch(IOException ex) {
      System.err.println(ex);
    } finally {
      try { sock.close(); } catch(Exception e) { } 
    }
  }
  
  /**
    */
  private void service(CacheRequest req, CacheResponse resp) {
    if (req.hasMethod("UPDATE")) {
      doUpdate(req, resp);
    } else if (req.hasMethod("LOOKUP")) {
      doLookUp(req, resp);
    } else if (req.hasMethod("FLUSH")) {
      doFlush(req, resp);
    } else {
      resp.setStatus("302");
    }
  }

  /**
    */
  private void doUpdate(CacheRequest req, CacheResponse resp) {
    CacheCertificate cert = req.getCertificate();
      
    if (cert == null) {
      resp.setStatus("301");
      return;
    }

    String name = req.getRootResourceName();
    Cache cache = Cache.getCache(name);

    synchronized(cache) {
      File dir = new File(new File(baseDir, name), req.getDirName());

      if (!dir.isDirectory() && !dir.mkdirs()) {
        resp.setStatus("401");
        return;
      } else if (!dir.canWrite()) {
        resp.setStatus("407");
        return;
      }

      String key = req.getCertHash();
      File certFile = new File(dir, key);

      if (certFile.isDirectory()) {
        resp.setStatus("403");
        return;
      } 

      boolean overwrite = certFile.exists();

      if (overwrite && !certFile.canWrite()) {
        resp.setStatus("405");
        return;
      } 

      try {
        cert.writeToFile(certFile);
        cache.addCacheEntry(
          new CacheEntry(key, cert.getNotAfter(), req.getDirName())
        );
        resp.setStatus(overwrite? "102" : "101");
      } catch(IOException ex) {
        resp.setStatus("201");
      }
    }
  }

  /**
    */
  private void doLookUp(CacheRequest req, CacheResponse resp) {
    String name = req.getRootResourceName();
    Cache cache = Cache.getCache(name);
      
    synchronized(cache) {
      String key = req.getCertHash();
      CacheEntry e = cache.getCacheEntry(key);
      boolean found = (e != null);

      if (found && !e.checkValidity()) {
        resp.setStatus("205");
        return;
      }

      File dir = new File(new File(baseDir, name), req.getDirName());

      if (found && !dir.isDirectory()) {
        resp.setStatus(dir.isFile() ? "402" : "400");
        return;
      }

      File certFile = new File(dir, key);

      if (found && !certFile.isFile()) {
        resp.setStatus("404");
        return;
      } else if (found && !certFile.canRead()) {
        resp.setStatus("406");
        return;
      }

      try {
        CacheCertificate cert = new CacheCertificate();

        cert.readFromFile(certFile);

        if (found) {
          resp.setCertificate(cert);
          resp.setStatus("101");
        } else if (cert.checkValidity()) {
          resp.setCertificate(cert);
          resp.setStatus("103");
          cache.addCacheEntry(
            new CacheEntry(key, cert.getNotAfter(), req.getDirName())
          );
        } else {
          certFile.delete();
          resp.setStatus("205");
        }
      } catch(FileNotFoundException ex) {
        resp.setStatus("204");
      } catch(IOException ex) {
        resp.setStatus("202");
      } catch(ParseException ex) {
        resp.setStatus("203");
      }
    }
  }

  /**
    */
  private void doFlush(CacheRequest req, CacheResponse resp) {
    String name = req.getRootResourceName();
    Cache cache = Cache.getCache(name);
    
    synchronized(cache) {
      File dir = new File(new File(baseDir, name), req.getDirName());

      if (!dir.exists()) {
        resp.setStatus("104");
        return;
      } if (dir.isFile()) {
        resp.setStatus("402");
        return;
      }
      
      String key = req.getCertHash();
      
      if (key.equalsIgnoreCase("ALL")) {
        resp.setStatus(deleteDirectory(cache, dir) ? "101" : "207");
      } else { 
        CacheEntry e = cache.getCacheEntry(key);
        boolean found = (e != null);
        File certFile = new File(dir, key);

        if (certFile.isDirectory()) {
          resp.setStatus("402");
        } else if (!certFile.isFile()) {
          resp.setStatus(found ? "404" : "104");
        } else if (found) { 
          cache.removeCacheEntry(e);
          resp.setStatus(certFile.delete() ? "101" : "206");
        } else {
          resp.setStatus(certFile.delete() ? "103" : "206");
        }
      }
    }
  }
   
  /**
    *
    */
  private boolean deleteDirectory(Cache cache,  File dir) {
    File []files = dir.listFiles();

    for (int i = 0; i < files.length; ++i) {
      if (files[i].isDirectory()) {
        deleteDirectory(cache, files[i]); 
      } else {
        CacheEntry e = cache.getCacheEntry(files[i].getName());
        
        if (e != null) {
          cache.removeCacheEntry(e);
        }

        files[i].delete();
      }
    }

    return dir.delete();
  }
}
