package lbl.akenti.servers.ResDefServer;

import com.sun.net.ssl.KeyManagerFactory;
import com.sun.net.ssl.SSLContext;
import com.sun.net.ssl.TrustManagerFactory;
import com.sun.net.ssl.internal.ssl.Provider;
import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.net.InetAddress;
import java.net.ServerSocket;
import java.net.Socket;
import java.security.GeneralSecurityException;
import java.security.KeyStore;
import java.security.SecureRandom;
import java.security.Security;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;
import java.text.ParseException;
import java.util.Enumeration;
import java.util.Properties;
import java.util.StringTokenizer;
import javax.net.ServerSocketFactory;
import javax.net.ssl.SSLServerSocket;
import lbl.akenti.certs.AkentiCertificate;
import lbl.akenti.certs.AkentiCertificateFactory;
import lbl.akenti.certs.PolicyCertificate;
import lbl.akenti.certs.elements.CertificateAuthorityInfo;
import lbl.akenti.util.*;
import lbl.akenti.util.P8KeyStore;
import lbl.akenti.util.P8Reader;

/**
 * This http server handles GET requests in secured and none secured mode.
 *
 */
public class ResourceDefinitionServer implements AkentiConstants{
  ServerSocket serverSocket;
  
  /**
   * Initializes member variable serverSocket.
   *
   * @exception ConfigurationException is thrown if getting from createServerSocket method.
   * @exception IOException is thrown if getting from createServerSocket method.
   * @exception GeneralSecurityException is thrown if getting from createServerSocket method.
   */
  public ResourceDefinitionServer()
  throws ConfigurationException, IOException, GeneralSecurityException, ParseException {
    serverSocket = createServerSocket();
  }
  
  /**
   * Creates a secured or none secured ServerSocket.
   *
   * @exception ConfigurationException is thrown if either Security, Port,
   *   CredentialFileName password, or CasFileName property retrieved from System is empty.
   * @exception IOException is thrown if error occured when retrieving properties from
   *   System.
   * @exception GeneralSecurityException is thrown when there's a java.security Exception.
   * @return a secured or none secured ServerSocket.
   */
  private ServerSocket
  createServerSocket()
  throws ConfigurationException, IOException, GeneralSecurityException,
  ParseException {
    String portString = System.getProperty("ResDefPort", ResDefPort);
    
    for (int i = 0; i < portString.length(); i++) {
      if (!Character.isDigit(portString.charAt(i))) {
        throw new ConfigurationException(
        "Invalid <i>Port</i> property in config file"
        );
      }
    }
    
    int port = Integer.parseInt(portString);
    
    InetAddress address = InetAddress.getLocalHost();
    String hostName = address.getHostName();
    String security = System.getProperty("ResDefSSL", ResDefSSL);
    
    if (security.equalsIgnoreCase("OFF")) {
      System.out.println(
      "Server is running at " + hostName + " port " + port + " in unsecured mode"
      );
      
      return new ServerSocket(port);
    } else if (security.equalsIgnoreCase("ON")) {
      KeyStore ks = new P8KeyStore();
      
      ks.load(null, null);
      
      String serverKeyFileName = System.getProperty("ResDefServerKey");
      
      if (serverKeyFileName == null) {
        throw new ConfigurationException(
        "Must provide a serverKeyFileName property in config file"
        );
      }
      
      String serverCertFileName = System.getProperty("ResDefServerCert");
      
      if (serverCertFileName == null) {
        throw new ConfigurationException(
        "Must provide a ResDefServerCert property in config file"
        );
      }
      
      P8Reader p8Reader = new P8Reader();
      
      p8Reader.readPrivateKey(serverKeyFileName);
      p8Reader.readX509Chain(serverCertFileName);
      ks.setKeyEntry(p8Reader.getSubjectDN().toString(),
      p8Reader.getPrivateKey(),
      null,
      p8Reader.getX509CertificateChain());
      
      CertificateFactory cfac = CertificateFactory.getInstance("X.509");
      
      RootResourceParser parser = RootResourceParser.getInstance();
      Enumeration resourceNames = parser.getRootResources();
      
      for (; resourceNames.hasMoreElements(); ) {
        String rootName = (String) resourceNames.nextElement();
        String fileName = parser.getRootFullPath(rootName) + rootName + "/" +
        System.getProperty("PolicyFileName", PolicyFileName);
        
        FileInputStream fis = new FileInputStream(fileName);
        AkentiCertificate tempPolCert = AkentiCertificateFactory.initCertFromStream(
        "PEM", fis
        );
        
        if (!tempPolCert.getType().equals("Policy")) {
          throw new ConfigurationException("Invalid Policy file found in " + fileName);
        }
        
        PolicyCertificate policyCertificate = (PolicyCertificate)tempPolCert;
        
        CertificateAuthorityInfo[]
        authorities = policyCertificate.getCertificateAuthorityInfos();
        
        for (int i = 0; i < authorities.length; i++) {
          X509Certificate[] chain = authorities[i].getChain();
          
          for (int j = 0; j < chain.length; j++) {
            String alias = chain[j].getSubjectDN().toString();
            
            ks.setCertificateEntry(alias, chain[j]);
          }
        }
      }
      
      KeyManagerFactory kfac = KeyManagerFactory.getInstance("SunX509");
      
      kfac.init(ks, null);
      
      TrustManagerFactory tfac = TrustManagerFactory.getInstance("SunX509");
      
      tfac.init(ks);
      
      SSLContext ctx = SSLContext.getInstance("TLS");
      
      File f = new File(".rand");
      SecureRandom random;
      
      random = SecureRandom.getInstance("SHA1PRNG");
      byte seed[] = null;
      
      try {
        if (f.exists()) {
          FileInputStream fis = new FileInputStream(f);
          byte temp[] = new byte[20];
          
          fis.read(temp);
          seed = temp;
        }
      } catch(IOException ex) {
      }
      
      if (seed == null) {
        System.out.println("this will take some time...");
        seed = random.generateSeed(20);
        
        try {
          FileOutputStream fos = new FileOutputStream(f);
          fos.write(seed);
          fos.flush();
          fos.close();
          
          System.out.println("A .rand file is saved into this directory");
          
        } catch(IOException ex) {
        }
      } else {
        random.setSeed(seed);
      }
      
      ctx.init( kfac.getKeyManagers(), tfac.getTrustManagers(), random);
      
      ServerSocketFactory fac = ctx.getServerSocketFactory();
      SSLServerSocket SSLServerSocket = (SSLServerSocket) fac.createServerSocket(port);
      
      SSLServerSocket.setNeedClientAuth(true);
      System.out.println(
      "Server is running at " + hostName + " port " + port + " in secured mode"
      );
      return SSLServerSocket;
    } else {
      throw new ConfigurationException("Security is neither ON nor OFF");
    }
  }
  
  /**
   * Creates a thread to handle each incoming client Socket.
   */
  public void start() throws IOException {
    while(true) {
      Socket socket = serverSocket.accept();
      HttpHelper helper = new HttpHelper(socket);
      Thread thread = new Thread(helper);
      
      thread.start();
    }
  }
  
  /**
   * Starts a new ResDefServer if the DocumentRoot property in the configuration
   * file specified at the command line is not empty.
   */
  public static void main(String argv[]) {
    try {
      if (argv.length == 0) {
        throw new Exception("Must provide a config file");
      }
      
      Security.addProvider(new Provider());
      
      ConfFileParser.loadFromFile(argv[0]);
      
      RootResourceParser parser = RootResourceParser.getInstance();
      
      parser.parseRootResource(argv[0]);
            
      new ResourceDefinitionServer().start();
      
    } catch(ConfigurationException ex) {
      ex.printStackTrace();
    } catch(GeneralSecurityException ex) {
      ex.printStackTrace();
    } catch (IOException e) {
      e.printStackTrace();
    } catch (Exception e) {
      e.printStackTrace();
    }
  }
}

/**
 * This is a simple server socket helper that parses request, calls HttpHandler for
 * handling GET request, and sends message to the client with the right http header.
 */
class HttpHelper implements Runnable {
  private Socket socket;
  
  /**
   * Initializes socket.
   *
   * @param socket client's Socket.
   * @exception IllegalArgumentException is thrown if socket is null.
   */
  HttpHelper(Socket socket) {
    if (socket == null) {
      throw new IllegalArgumentException();
    }
    
    this.socket = socket;
  }
  
  /**
   * Parses the request, calls HttpHandler for handling GET request, and sends message
   * to the client with the right http header.
   */
  public void run() {
    try {
      HttpRequest request = new HttpRequest();
      ByteArrayOutputStream baos = new ByteArrayOutputStream();
      HttpResponse response = new HttpResponse();
      
      response.setOutputStream(baos);
      try {
        parseRequest(socket.getInputStream(), request);
        
        if (request.getMethod().compareToIgnoreCase("GET") == 0) {
          HttpHandler handler = new HttpHandler();
          
          handler.doGet(request, response);
        } else {
          response.setStatusCode(405);
        }
      } catch (BadRequestException e) {
        if (response.getStatusCode() == -1) {
          response.setStatusCode(400);
        }
      } catch (ForbiddenException e) {
        if (response.getStatusCode() == -1) {
          response.setStatusCode(403);
        }
      } catch (MovedPermanentlyException e) {
        if (response.getStatusCode() == -1) {
          response.setStatusCode(301);
        }
      }
      
      writeResponse(socket.getOutputStream(), request, response);
    } catch (IOException e) {
      e.printStackTrace();
    } catch (ConfigurationException e) {
      e.printStackTrace();
    }
    
    try {
      socket.close();
    } catch (IOException e) {
    }
  }
  
  /**
   * Parses the request including header and body.
   *
   * @param in client Socket's InputStream.
   * @param request HttpRequest object to be initialized.
   * @exception IOException is thrown if there's a problem when reading from InputStream.
   * @exception BadRequestException is thrown if the client's http header is incorrect.
   * @exception ForbiddenException is thrown if URI contains a "~" or a "." others than
   *   the four .gif files in /icons/ directory.
   */
  private void parseRequest(InputStream in, HttpRequest request)
  throws IOException, BadRequestException, ForbiddenException {
    BufferedReader br = new BufferedReader(new InputStreamReader(in), 1);
    String line  = br.readLine();
    
    if (line == null) {
      throw new BadRequestException("Invalid HTTP request.");
    }
    
    StringTokenizer tokenizer  = new StringTokenizer(line);
    
    ensureMoreTokens(tokenizer);
    request.setMethod(tokenizer.nextToken());
    ensureMoreTokens(tokenizer);
    line = tokenizer.nextToken();
    
    if (line.indexOf("~") > 0 || line.indexOf("..") > 0) {
      throw new ForbiddenException();
    }
    
    request.setURI(line);
    
    int contentLength = 0;
    
    while ((line = br.readLine()) != null && !line.equals("")) {
      tokenizer = new StringTokenizer(line);
      
      String token =  tokenizer.nextToken();
      
      if (token.equalsIgnoreCase("Content-Type:")) {
        ensureMoreTokens(tokenizer);
        request.setContentType(tokenizer.nextToken());
      }
      
      if (token.equalsIgnoreCase("Content-Length:")) {
        ensureMoreTokens(tokenizer);
        token = tokenizer.nextToken();
        
        for (int i = 0; i < token.length(); i++) {
          if (!Character.isDigit(token.charAt(i))) {
            throw new BadRequestException("Invalid Content-Length");
          }
        }
        
        contentLength = Integer.parseInt(token);
      }
    }
    
    if (line == null) {
      throw new BadRequestException();
    }
    
    request.setContentLength(contentLength);
    
    char[] buffer = new char[contentLength];
    
    br.read(buffer);
    
    request.setInputStream(
    new ByteArrayInputStream(new String(buffer).getBytes())
    );
  }
  
  /**
   * Responses to the client request.
   *
   * @param out OutputStream to write to the client.
   * @param response contains response info.
   * @exception IOException is thrown if there is a problem when writing to the client.
   */
  private void writeResponse(OutputStream out, HttpRequest request, HttpResponse response)
  throws IOException {
    final String serverLine = "Server: ResDefServer (Unix)\r\n";
    final String genericContentTypeLine ="Content-type: text/html\r\n";
    
    if (response.getStatusCode() == 301) {
      String entityBody = "<HTML><HEAD><TITLE>301 Moved Permanently</TITLE>" +
      "</HEAD><BODY><H1>Moved Permanently</H1>" +
      "The document has moved <A HREF=\"" + request.getURI() +
      "/\">here</A><P></BODY></HTML>";
      out.write("HTTP/1.0 301 Moved Permanently\r\n".getBytes());
      out.write(serverLine.getBytes());
      out.write(genericContentTypeLine.getBytes());
      out.write(("Content-Length: " + entityBody.length() + "\r\n").getBytes());
      out.write("\r\n".getBytes());
      out.write(entityBody.getBytes());
    } else if (response.getStatusCode() == 403) {
      String entityBody = "<HTML><HEAD><TITLE>403 Forbidden</TITLE>" +
      "</HEAD><BODY><H1>Forbidden</H1>" +
      "The request was forbidden." +
      "<P></BODY></HTML>";
      out.write("HTTP/1.0 403 Forbidden\r\n".getBytes());
      out.write(serverLine.getBytes());
      out.write(genericContentTypeLine.getBytes());
      out.write(("Content-Length: " + entityBody.length() + "\r\n").getBytes());
      out.write("\r\n".getBytes());
      out.write(entityBody.getBytes());
    } else if (response.getStatusCode() == 400) {
      String entityBody = "<HTML><HEAD><TITLE>400 Bad Request</TITLE>" +
      "</HEAD><BODY><H1>Bad Request</H1>" +
      "The server could not understand the request." +
      "<P></BODY></HTML>";
      out.write("HTTP/1.0 400 Bad Request\r\n".getBytes());
      out.write(serverLine.getBytes());
      out.write(genericContentTypeLine.getBytes());
      out.write(("Content-Length: " + entityBody.length() + "\r\n").getBytes());
      out.write("\r\n".getBytes());
      out.write(entityBody.getBytes());
    } else if(response.getStatusCode() == 405) {
      String entityBody = "<HTML><HEAD><TITLE>405 Method Not Allowed</TITLE>" +
      "</HEAD><BODY><H1>Method Not Allowed</H1>" +
      "The requested method is not supported by this URL." +
      "<P></BODY></HTML>";
      out.write("HTTP/1.0 405 Method Not Allowed\r\n".getBytes());
      out.write(serverLine.getBytes());
      out.write(genericContentTypeLine.getBytes());
      out.write(("Content-Length: " + entityBody.length() + "\r\n").getBytes());
      out.write("\r\n".getBytes());
      out.write(entityBody.getBytes());
      
    }else {
      out.write("HTTP/1.0 200 OK\r\n".getBytes());
      out.write(serverLine.getBytes());
      out.write(("Content-type: " + response.getContentType() +
      "\r\n").getBytes());
      
      ByteArrayOutputStream
      baos = (ByteArrayOutputStream) response.getOutputStream();
      
      out.write(("Content-Length: " + baos.size() + "\r\n").getBytes());
      out.write("\r\n".getBytes());
      out.write(baos.toByteArray());
    }
  }
  
  /**
   * Ensures that there is at least one more token.
   *
   * @param tokenizer StringTokenizer to be checked.
   * @exception BadRequestException is thrown if no more tokens.
   */
  private void ensureMoreTokens(StringTokenizer tokenizer) throws BadRequestException {
    if (!tokenizer.hasMoreTokens()) {
      throw new BadRequestException("Invalid HTTP request.");
    }
  }
}