package lbl.akenti.tools;

import com.sun.net.ssl.internal.ssl.Provider;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.OutputStream;
import java.security.PrivateKey;
import java.security.Security;
import java.util.Properties;
import lbl.akenti.certs.AkentiCertificate;
import lbl.akenti.certs.AkentiCertificateFactory;
import lbl.akenti.certs.AttributeCertGenerator;
import lbl.akenti.certs.AttributeCertificate;
import lbl.akenti.certs.PolicyCertGenerator;
import lbl.akenti.certs.PolicyCertificate;
import lbl.akenti.certs.UseCondCertGenerator;
import lbl.akenti.certs.UseCondCertificate;
import lbl.akenti.certs.elements.AkentiIssuer;
import lbl.akenti.certs.elements.AkentiPrincipal;
import lbl.akenti.certs.elements.CertificateAuthorityInfo;
import lbl.akenti.certs.elements.Condition;
import lbl.akenti.certs.elements.Directory;
import lbl.akenti.util.ArgumentParser;
import lbl.akenti.util.P12Reader;
import lbl.akenti.util.UTCTime;
import lbl.akenti.util.Util;


/**
 * example: CertGen -k mykeyfile -p mypasswordfile -i xmlCertFile
 * options:
 * -k arg        - keyfile in p12 format
 * -p arg        - file where passphrase is stored
 * -i arg        - xml input file
 * [-o arg]      - friendly name to store file
 * [-d arg]      - number of days
 * [-v]          - verbose
 * [-h]          - prints this message
 *
 * @author Tommaso Sciortino
 */
public class CertGen {
  /**
   * Program to generate and sign a certificate from an XML input file.
   *
   * Note: The xml input file you provide must contain a valid xml
   * encoded certificate according to the akenti certificate schema.
   * Otherwise the certificate parser would refuse it.
   * Edit existing certificates to avoid problems. And don't forget to
   * leave the old signature element intact.
   * @param args Command line arguments.
   */
  public static void main(String[] args) {
    String keyFileName = null;
    String passwordFileName = null;
    String outCertFile = null;
    String inCertFile = null;
    int period = -1;
    
    try{
      ArgumentParser parser = new ArgumentParser(args);
      
      if (args.length == 0) {
        printUsage();
        System.exit(2);
      }
      
      parser.parseArgumentTokens(new char[] {'d', 'k', 'p', 'o', 'i', 'h'});
      
      keyFileName = parser.getOptionArg('k');
      passwordFileName = parser.getOptionArg('p');
      outCertFile = parser.getOptionArg('o');
      inCertFile = parser.getOptionArg('i');
      if (parser.getOption ('d')) {
	period = new Integer(parser.getOptionArg('d')).intValue();
      }
      if (parser.getOption('h')) {
        printUsage();
        System.exit(2);
      } else if (keyFileName == null || keyFileName.equals("")) {
        System.err.println("Must provide a p12 file");
        System.exit(2);
      } else if (passwordFileName == null || passwordFileName.equals("")) {
        System.err.println("Must provide a password");
        System.exit(2);
      } else if (inCertFile  == null || inCertFile.equals("")) {
        System.err.println("Must provide an xml input file");
        System.exit(2);
      }
      
    } catch (IOException ex) {
      System.err.println(ex.getMessage());
      printUsage();
      System.exit(2);
    }
    
    try {
      Security.addProvider(new Provider());
      
      P12Reader p12 = new P12Reader();
      
      File passwordFile = new File(passwordFileName);
      if (!passwordFile.canRead()) {
        System.err.println("Password cannot be read from file: " + passwordFileName);
      }
      
      BufferedReader in = new BufferedReader(new FileReader(passwordFile));
      String buffer = null;
      StringBuffer password = new StringBuffer();
      while ((buffer = in.readLine()) != null) {
        password.append(buffer);
      }
      in.close();

      System.out.println("Loading p12 file from: " + keyFileName);
      p12.readP12File(keyFileName, password.toString());
      
      System.err.println("XML file being parsed: " + new File(inCertFile).getAbsolutePath());
      
      AkentiCertificate inCert = AkentiCertificateFactory.initCertFromFile(
      "XML", inCertFile
      );
      
      String type = inCert.getType();
      AkentiCertificate outCert = null;
      
      if (type.equals("Attribute")) {
        outCert = generateAttributeCertificate((AttributeCertificate) inCert, p12, period);
      } else if (type.equals("UseCondition")) {
        outCert = generateUseCondCertificate((UseCondCertificate) inCert, p12, period);
      } else if (type.equals("Policy")) {
        outCert = generatePolicyCertificate((PolicyCertificate)inCert, p12, period);
      } else {
        throw new IOException("Can't handle type " + type);
      }
      
      // Don't add hash to file name for policy certificates
      if (!type.equals("Policy")){
        outCertFile += '.' + Util.createHashName(outCert.getHashString()) + ".";
        outCertFile = Util.createFileName(outCertFile); 
      }
      
      OutputStream fos = null;

      if (outCertFile != null) {
        System.err.println("Writing cert to " + new File(outCertFile).getAbsolutePath());
      
        fos  = new FileOutputStream(new File(outCertFile));
      } else {
        System.err.println("Writing cert to screen");
        fos = System.out;
      }

      AkentiCertificateFactory.writeCertToStream("XML", outCert, fos);
      
      fos.close();
      System.exit(0);
      
    } catch(RuntimeException ex) {
      System.err.println("SHOULD NEVER HAPPEN");
      ex.printStackTrace();
      System.exit(3);
    } catch(Exception ex) {
      ex.printStackTrace();
      System.exit(3);
    }
  }
  
  /**
   * Generate the xml file for the AttributeCertificate.
   */
  private static AttributeCertificate
  generateAttributeCertificate(AttributeCertificate inCert, P12Reader p12, int period)
  throws Exception {
    AttributeCertGenerator gen = new AttributeCertGenerator();
    
    gen.setVersionNumber(2);
    gen.setUniqueID(computeUniqueID(inCert));
    
    AkentiPrincipal issuer = new AkentiPrincipal(p12.getSubjectDN(), p12.getIssuerDN());
    
    gen.setIssuer(new AkentiIssuer(issuer));
    
    UTCTime[] temp = computeValidityTimes(inCert, period);
    
    gen.setNotBefore(temp[0]);
    gen.setNotAfter(temp[1]);
    
    gen.setPrincipal(inCert.getPrincipal());
    gen.setAttribute(inCert.getAttribute());
    gen.setValue(inCert.getValue());
    
    Condition cond = inCert.getCondition();
    
    if (cond != null) {
      gen.setCondition(inCert.getCondition());
    }
    
    PrivateKey privateKey = p12.getPrivateKey();
    
    gen.sign("MD5withRSA", privateKey);
    return (AttributeCertificate) gen.generate();
  }
  
  /**
   * Generate the xml file for the UseCondCertificate.
   */
  private static UseCondCertificate
  generateUseCondCertificate(UseCondCertificate inCert, P12Reader p12, int period)
  throws Exception {
    UseCondCertGenerator gen = new UseCondCertGenerator();
    
    gen.setVersionNumber(2);
    gen.setUniqueID(computeUniqueID(inCert));
    
    AkentiPrincipal issuer = new AkentiPrincipal(p12.getSubjectDN(), p12.getIssuerDN());
    
    gen.setIssuer(new AkentiIssuer(issuer));
    
    UTCTime[] temp = computeValidityTimes(inCert, period);
    
    gen.setNotBefore(temp[0]);
    gen.setNotAfter(temp[1]);
    gen.setResource(inCert.getResource());
    gen.setCritical(inCert.isCritical());
    gen.setLocalScope(inCert.isScopeLocal());
    gen.setCondition(inCert.getCondition());
    gen.setActions(inCert.getActions());
    
    PrivateKey privateKey = p12.getPrivateKey();
    
    gen.sign("MD5withRSA", privateKey);
    return (UseCondCertificate) gen.generate();
  }
  
  /**
   * Generate the xml file for the PolicyCertificate.
   */
  private static PolicyCertificate
  generatePolicyCertificate(PolicyCertificate inCert,P12Reader p12, int period)
  throws Exception {
    
    PolicyCertGenerator gen = new PolicyCertGenerator();
    
    gen.setVersionNumber(2);
    gen.setUniqueID(computeUniqueID(inCert));
    
    AkentiPrincipal issuer = new AkentiPrincipal(p12.getSubjectDN(), p12.getIssuerDN());
    
    gen.setIssuer(new AkentiIssuer(issuer));
    
    UTCTime[] temp = computeValidityTimes(inCert, period);
    
    gen.setNotBefore(temp[0]);
    gen.setNotAfter(temp[1]);
    gen.setResource(inCert.getResource());
    
    CertificateAuthorityInfo[] cais = inCert.getCertificateAuthorityInfos();
    
    if (cais.length != 0) {
      gen.setCertificateAuthorityInfos(cais);
    }
    
    gen.setUseCondIssuerGroups(inCert.getUseCondIssuerGroups());
    
    Directory[] attrDirs = inCert.getAttributeDirectories();
    
    if (attrDirs.length != 0) {
      gen.setAttributeDirectories(attrDirs);
    }
    
    gen.setCacheTime(inCert.getCacheTime());
    
    PrivateKey privateKey = p12.getPrivateKey();
    
    gen.sign("MD5withRSA", privateKey);
    return (PolicyCertificate) gen.generate();
  }
  
  /**
   * Uses the period supplied by the user otherwise it Uses the
   * <code>ValidityTimes</code> from the supplied certificate.
   */
  private static UTCTime[]
  computeValidityTimes(AkentiCertificate cert, int period)
  throws Exception {
    UTCTime now = new UTCTime();
    
    long l1 = cert.getNotBefore().getTime();
    long l2 = cert.getNotAfter().getTime();
    
    if (period < 0) {
      UTCTime notBefore = cert.getNotBefore();
      
      if (notBefore.compareTo(now) < 0) {
        notBefore = now;
      }
      
      UTCTime notAfter = cert.getNotAfter();
      
      if (notBefore.compareTo(notAfter) >= 0) {
        throw new Exception("Bad expiration time");
      }
      
      return new UTCTime[] { notBefore, notAfter};
    }
    
    return new UTCTime[] { now, new UTCTime(now, 1000L * 60 * 60 * 24 * period) };
  }
  
  /**
   * Computes the unique id.
   */
  private static String computeUniqueID(AkentiCertificate cert) {
    return Util.createUniqueID(Util.createHashName(cert.getHashString()));
  }
  
  private static void printUsage() {
    System.err.println("example: CertGen -k mykeyfile -p mypasswordfile -i xmlCertFile");
    System.err.println("options:");
    System.err.println("  -k   keyfile in p12 format");
    System.err.println("  -p   file where passphrase is stored");
    System.err.println("  -i   xml input file");
    System.err.println("  [-o] friendly name to store file");
    System.err.println("  [-d] number of days");
    System.err.println("  [-v] verbose");
    System.err.println("  [-h] prints this message");
  }
}
