/*
 * RemoteCheckAccess.java
 *
 * Created on July 21, 2004, 3:00 PM
 */

package lbl.akenti.tools;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.PrintStream;
import java.net.URL;
import java.security.cert.X509Certificate;
import lbl.akenti.certs.AkentiCertificate;
import lbl.akenti.certs.AkentiCertificateFactory;
import lbl.akenti.certs.CapabilityCertificate;
import lbl.akenti.certs.elements.AkentiPrincipal;
import lbl.akenti.certs.elements.ConditionalActions;
import lbl.akenti.messaging.CertificateInfo;
import lbl.akenti.messaging.CheckAccessRequest;
import lbl.akenti.messaging.CheckAccessResponse;
import lbl.akenti.messaging.PolicyInfo;
import lbl.akenti.messaging.RestrictedAttribute;
import lbl.akenti.protocolHandler.RemoteProtocolHandler;
import lbl.akenti.util.AkentiConstants;
import lbl.akenti.util.ArgumentParser;
import lbl.akenti.util.ConfFileParser;
import lbl.akenti.util.X509CertReader;

/**
 *  example: CheckAccess -f conf.txt -r "R1" -x cert -R read -R write
 *   -F arg             - akenti engine config file
 *   -r arg             - resource
 *   [-x arg]           - user's x509 cert chain
 *   [-u arg -c arg]    - user's DN and user's CADN
 *   [-R arg]*          - a specific action(s)
 *   [-A arg -V arg]*   - attribute/value pair
 *   [-C]               - show capability cert only
 *   [-P]               - show all certs used in making the decision
 *   [-o arg]           - certs' output file (defaults to stdout)
 *   [-q]               - don't show cap in pretty text (the default)
 *   [-v]               - verbose
 *   [-h]               - prints this message
 * *multiple instances of this flag allowed
 *
 * @author Tommaso Sciortino
 */
public class RemoteCheckAccess implements AkentiConstants {
  
  /**
   * Use this program to remotely collect a user's access to a resource.
   * @param args Command line argument.
   */
  public static void main(String[] args) {
    String confFileName = null;
    String resource = null;
    String x509CertFileName = null;
    String dn = null;
    String cadn = null;
    String[] desiredActions = null;
    String[] attribute = null;
    String[] value = null;
    String output = null;
    
    boolean detailed = false;
    boolean concise = false;
    boolean verbose = false;
    
    try{
      ArgumentParser parser = new ArgumentParser(args);
      
      if (args.length == 0) {
        printUsage();
        System.exit(2);
      }
      
      char[] flags = new char[] {'F', 'r', 'x', 'u', 'c', 'R', 'A', 'V', 'o', 'P', 'q', 'v', 'h'};
      parser.parseArgumentTokens(flags);
      
      confFileName = parser.getOptionArg('F');
      resource = parser.getOptionArg('r');
      x509CertFileName = parser.getOptionArg('x');
      dn = parser.getOptionArg('u');
      cadn = parser.getOptionArg('c');
      
      desiredActions = parser.getOptionArgs('R');
      
      attribute = parser.getOptionArgs('A');
      value = parser.getOptionArgs('V');
      
      output = parser.getOptionArg('o');
      
      detailed = parser.getOption('P');
      concise = parser.getOption('q');
      verbose = parser.getOption('v');
      
      if (parser.getOption('h')) {
        printUsage();
        System.exit(2);
      } else if (confFileName == null || confFileName.equals("")) {
        System.err.println("Must provide a config file");
        System.exit(2);
      } if ( (x509CertFileName == null || x509CertFileName.equals("") ) &&
      (dn == null || dn.equals("") || cadn == null || cadn.equals("")) ) {
        System.err.println("Must proide a user's x509 certificate or DN/CADN pair");
        System.exit(2);
      } if (parser.getOption('x') && (parser.getOption('u') || parser.getOption('c')) ) {
        System.err.println("Must proide either a user's x509 certificate or DN/CADN pair but not both");
        System.exit(2);
      } if (attribute.length != value.length) {
        System.err.println("The number of attributes must match the number of values.");
        System.exit(2);
      } else if (resource == null || resource.equals("")) {
        System.err.println("Must provide a resource");
        System.exit(2);
      }
      
    } catch (IOException ex) {
      System.err.println(ex.getMessage());
      printUsage();
      System.exit(2);
    }
    
    try {
      ConfFileParser.loadFromFile(confFileName);
     
      String host = System.getProperty("AkentiHost", AkentiHost);
      String sslMode = System.getProperty("SslMode", SslMode);
      int port = 0;
      URL url;
      
      if (sslMode.equals("on")) {
        SSLUtil.initSSL();
        port = new Integer(System.getProperty("AkentiPort", AkentiPortSSL)).intValue();
        url = new URL("https", host, port,"/");
      } else {
        port = new Integer(System.getProperty("AkentiPort", AkentiPort)).intValue();
        url = new URL("http", host, port,"/");
      }
      
      RestrictedAttribute[] resAttrs = new RestrictedAttribute[attribute.length];
      for (int i=0; i < attribute.length; i++) {
        resAttrs[i] = new RestrictedAttribute(attribute[i],value[i]);
      }
      
      AkentiPrincipal user = null;
      
      if (x509CertFileName != null) {
        if (verbose) {
          System.err.println("Reading user's cert chain from " + x509CertFileName);
        }
        
        X509CertReader x509Reader = new X509CertReader();
        X509Certificate x509Cert = x509Reader.readX509Cert(x509CertFileName);
        
        user = new AkentiPrincipal(x509Cert.getSubjectDN(), x509Cert.getIssuerDN());
      } else {
        user = new AkentiPrincipal(dn,cadn);
      }
      
      RemoteProtocolHandler eng = new RemoteProtocolHandler(url);
      CheckAccessRequest req = new CheckAccessRequest(RemoteProtocolHandler.getID(),
      user,resource,desiredActions,resAttrs,detailed);
      CheckAccessResponse resp = null;
      
      try {
        resp = eng.process(req);
      } catch(Exception ex) {
        System.err.println("There was an error connecting to " + url.toString());
        System.exit(3);
      }
      
      if (verbose) {
        System.out.println(resp.getMessage());
      }
      
      PrintStream os = System.out;
      if (output != null) {
        os = new PrintStream(new FileOutputStream(output));
      }
      
      if (!detailed) {
        
        if (resp.hasCapabilityCertificate()) {
          CapabilityCertificate capCert = resp.getCapabilityCertificate();
          if (concise) {
            os.print(printActions(capCert.getUnConditionalActions()));
            os.print(printActions(capCert.getConditionalActions()));
          } else {
            AkentiCertificateFactory.writeCertToStream(AK_CERT_ENC_TYPE, capCert, os);
          }
        }
        
      } else {
        
        PolicyInfo[] policyInfos = resp.getPolicyInfos();
        for (int i = 0; i < policyInfos.length; ++i) {
          CertificateInfo[] certInfos = policyInfos[i].toCertificateInfosArray();
          for (int j = 0; j < certInfos.length; ++j) {
            if (certInfos[j].getAkentiCertificate() != null) {
              AkentiCertificateFactory.writeCertToStream(AK_CERT_ENC_TYPE, certInfos[j].getAkentiCertificate(), os);
            }
          }
        }
        
        AkentiCertificate[] akentiCerts = resp.getCertificates();
        
        for (int k = 0; k < akentiCerts.length; ++k) {
          AkentiCertificateFactory.writeCertToStream(AK_CERT_ENC_TYPE, akentiCerts[k], os);
        }
        
        if (resp.hasCapabilityCertificate()) {
          AkentiCertificateFactory.writeCertToStream(AK_CERT_ENC_TYPE, resp.getCapabilityCertificate(), os);
        }
        
      }
      
      os.println(resp.getMessage());
      
      if (resp.getMessage().getCode() < 200) {
        System.exit(0);
      } else {
        System.exit(1);
      }
    } catch(RuntimeException ex) {
      System.err.println("SHOULD NEVER HAPPEN");
      ex.printStackTrace();
      System.exit(3);
    } catch(Exception ex) {
      ex.printStackTrace();
      System.exit(3);
    }
    
  }
  
  /**
   * Prints the usage instructions for this program
   */
  static void printUsage() {
    System.err.println("example: CheckAccess -f conf.txt -r 'R1' -x cert -R read -R write");
    System.err.println("options:");
    System.err.println("  -F arg             - akenti engine config file");
    System.err.println("  -r arg             - resource");
    System.err.println("  [-x arg]           - user's x509 cert chain");
    System.err.println("  [-u arg -c arg]    - user's DN and user's CADN");
    System.err.println("  [-R arg]*          - a specific action(s)");
    System.err.println("  [-A arg -V arg]*   - attribute/value pair");
    System.err.println("  [-C]               - show capability cert only");
    System.err.println("  [-P]               - show all certs used in making the decision");
    System.err.println("  [-o arg]           - certs' output file (defaults to stdout)");
    System.err.println("  [-q]               - don't show cap in pretty text (the default)");
    System.err.println("  [-v]               - verbose");
    System.err.println("  [-h]               - prints this message");
    System.err.println("*multiple instances of this flag allowed");
  }
  
  static private String printActions(String[] input) {
    StringBuffer buf = new StringBuffer("Allowed actions: \n");
    
    if (input.length == 0) {
      buf.append("NONE\n");
      return buf.toString();
    }
    
    buf.append(input[0] + "\n");
    
    for (int i=1; i < input.length; i++) {
      buf.append(", " + input[i]);
    }
    
    buf.append("\n");
    
    return buf.toString();
  }
  
  static private String printActions(ConditionalActions[] input) {
    StringBuffer buf = new StringBuffer("Conditional actions are: \n");
    
    if (input.length == 0) {
      buf.append("NONE\n");
      return buf.toString();
    }
    
    buf.append("\n");
    
    for (int i=0; i < input.length; i++) {
      buf.append("    " + input[i].toLogString() + "\n");
    }
    
    buf.append("\n");
    
    return buf.toString();
  }
  
}