/*
 * ConvertAkentiCert.java
 *
 * Created on July 1, 2004, 3:39 PM
 */

package lbl.akenti.tools;

import java.io.IOException;
import java.security.cert.X509Certificate;
import lbl.akenti.certs.AkentiCertificate;
import lbl.akenti.certs.AkentiCertificateFactory;
import lbl.akenti.util.ArgumentParser;
import lbl.akenti.util.UTCTime;
import lbl.akenti.util.X509CertReader;
import java.util.Properties;

/**
 * example: VerifyAkentiCert -a mycertificate
 * options:
 * -a arg      - akenti certificate
 * [-i arg]    - issuer's x509 certificate
 * [-v]        - verbose
 * [-h]        - prints this message
 *
 * @author Tommaso Sciortino
 */
public class VerifyAkentiCert {
  
  /**
   * Use this program to check the following:
   * Check 1: akenti certificate has not expired
   * Check 2: akenti certificate signature is valid (optional).
   *
   * Note: A check is also made on the time validity of the signer's cert
   * if one is supplied.
   * @param args Command line argument.
   */  
  public static void main(String[] args) {
    String inFile = null;
    String x509File = null;
    boolean verbose = false;
    
    try{
      ArgumentParser parser = new ArgumentParser(args);
      
      if (args.length == 0) {
        printUsage();
        System.exit(2);
      }
      
      parser.parseArgumentTokens(new char[] {'a','i', 'v', 'h'} );
      
      inFile = parser.getOptionArg('a');
      x509File = parser.getOptionArg('i');
      verbose = parser.getOption('v');
      
      if (parser.getOption('h')) {
        printUsage();
        System.exit(2);
      } else if (inFile == null || inFile.equals("")) {
        System.err.println("You must specify a certificate");
        System.exit(2);
      }
    } catch (IOException ex) {
      System.err.println(ex.getMessage());
      printUsage();
      System.exit(2);
    }
    
    try {
      
      if (verbose) {
        System.err.println("Loading Certificate...");
      }
      
      AkentiCertificate cert = AkentiCertificateFactory.initCertFromFile(null,inFile);
      
      UTCTime now = new UTCTime();
      
      if(cert.checkValidity(now)){
        System.out.print("Akenti Certificate time is valid as of ");
      } else {
        System.out.print("Akenti Certificate time is not valid as of ");
      }
      
      System.out.println(now.toFriendlyString());
      
      if (x509File != null && !x509File.equals("")) {
        if (verbose) {
          System.err.println("Loading X509 Certificate....");
        }

        X509CertReader x509Reader = new X509CertReader();
        X509Certificate x509 = x509Reader.readX509Cert(x509File);
        
        UTCTime x509BeforeTime = new UTCTime(x509.getNotBefore());
        UTCTime x509AfterTime = new UTCTime(x509.getNotAfter());
        
        if(x509AfterTime.compareTo(now) != 1 || x509BeforeTime.compareTo(now) != -1){
          System.out.println("issuer's certificate is not valid as of " + now.toFriendlyString());
        } else if (!cert.verify(x509.getPublicKey())) {
          System.out.println("invalid signature");
        } else {
          System.out.println("signature verifies ok");
        }
      }
      
      System.exit(0);
    } catch(RuntimeException ex) {
      System.err.println("SHOULD NEVER HAPPEN");
      ex.printStackTrace();
      System.exit(3);
    } catch(Exception ex) {
      ex.printStackTrace();
      System.exit(3);
    }
    
  }
  /**
   * Prints the usage instructions for this program
   */  
  static void printUsage() {
    System.err.println("example: VerifyAkentiCert -a mycertificate");
    System.err.println("options:");
    System.err.println("  -a arg      - akenti certificate");
    System.err.println("  [-i arg]    - issuer's x509 certificate");
    System.err.println("  [-v]        - verbose");
    System.err.println("  [-h]        - prints this message");
  }
  
}
