package lbl.akenti.util;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.net.URL;
import java.net.URLConnection;
import java.net.URLStreamHandler;
import java.net.URLStreamHandlerFactory;
import java.security.cert.X509Certificate;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Hashtable;
import javax.naming.Context;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.directory.Attribute;
import javax.naming.directory.DirContext;
import javax.naming.directory.InitialDirContext;
import javax.naming.directory.SearchControls;
import javax.naming.directory.SearchResult;

public class LDAPSearchUtil {

    /**
     * Object scope * searches only the named object.
     */
    private static final int OBJECT_SCOPE = 0;

    /**
     * One level scope searches all entries at the level of the named object.
     */
    private static final int ONELEVEL_SCOPE = 1;

    /**
     * Sub-tree scope searches the entire sub-tree below the named object.
     */
    private static final int SUBTREE_SCOPE = 2;

    /**
     */
    private static final String PROVIDER = "com.sun.jndi.ldap.LdapCtxFactory";


    /**
     */
    private static final String AUTH_TYPE = "simple";

    /**
     */
    private LDAPSearchUtil() {
    }

    public static void init() {
        URL.setURLStreamHandlerFactory(new URLStreamHandlerFactory() {
            public URLStreamHandler createURLStreamHandler(String protocol) {
                if (protocol.equalsIgnoreCase("ldap")) {
                    return new URLStreamHandler() {
                        protected URLConnection
                                openConnection(URL url) throws IOException {
                            throw new IOException("open ldap connection unsupported");
                        }
                    };
                }

                return null;
            }
        });
    }

    public static String[] loadOrgUnits(URL url) throws NamingException {
        return loadOrgUnits(url, "ou=*");
    }

    public static String[] loadOrgUnits(URL url, String filter) throws NamingException {
        ArrayList results = find(url, filter, new String[]{"ou"}, true, SUBTREE_SCOPE);

        return (String[]) results.toArray(new String[0]);
    }

    public static String[] loadCns(URL url, String filter) throws NamingException {
        ArrayList results = find(url, filter, new String[]{"cn"}, true, SUBTREE_SCOPE);

        return (String[]) results.toArray(new String[0]);
    }

    public static X509Certificate[] loadX509Certs(URL url, String filter) throws NamingException {
        String[] attrs = new String[]{"usercertificate;binary", "cacertificate;binary"};

        return loadX509Certs(url, filter, attrs);
    }

    public static X509Certificate[] loadCAX509Certs(URL url, String filter) 
        throws NamingException {
        String[] attrs = new String[]{"cacertificate;binary"};

        return loadX509Certs(url, filter, attrs);
    }

    public static X509Certificate[] loadUSERX509Certs(URL url, String filter) 
        throws NamingException {
        String[] attrs = new String[]{"usercertificate;binary"};

        return loadX509Certs(url, filter, attrs);
    }

    private static X509Certificate[] loadX509Certs(URL url, String filter, String[] attrs) 
        throws NamingException {
        ArrayList results = find(url, filter, attrs, true, SUBTREE_SCOPE);
        X509Certificate[] certs = new X509Certificate[results.size()];
        X509CertReader reader = new X509CertReader();

        for (int i = 0; i < results.size(); ++i) {
            try {
                ByteArrayInputStream bais = new ByteArrayInputStream((byte[]) results.get(i));

                certs[i] = reader.readX509Cert(bais);
            } catch (RuntimeException ex) {
                throw ex;
            } catch (Exception ex) {
                // READ ON
            }
        }

        if (certs.length == 0) {
            throw new NamingException("did not find any certificates");
        }
        return certs;
    }

    public static String[] loadBases(URL url) throws NamingException {
        String[] attrs = new String[]{"namingContexts"};
        ArrayList results = find(url, "(objectclass=*)", attrs, true, OBJECT_SCOPE);

        return (String[]) results.toArray(new String[0]);
    }

    private static ArrayList find(URL url, String filter, String[] attrs, boolean follow, int scope)
        throws NamingException {
        if (!url.getProtocol().equalsIgnoreCase("ldap")) {
            throw new RuntimeException();
        }

        StringBuffer sb = new StringBuffer("ldap://");

        sb.append(url.getHost());
        sb.append(":");

        int port = url.getPort();

        if (port == -1) {
            sb.append(389);
        } else {
            sb.append(port);
        }

        Hashtable hTable = new Hashtable();

        hTable.put(Context.INITIAL_CONTEXT_FACTORY, PROVIDER);
        hTable.put(Context.PROVIDER_URL, sb.toString());
        hTable.put(Context.SECURITY_AUTHENTICATION, "simple");
        hTable.put(Context.SECURITY_PRINCIPAL, "");
        hTable.put(Context.SECURITY_CREDENTIALS, "");

        if (follow) {
            hTable.put(Context.REFERRAL, "follow");
        }

        DirContext ctx = new InitialDirContext(hTable);
        SearchControls constraints = new SearchControls();

        constraints.setSearchScope(scope);
        constraints.setReturningAttributes(attrs);

        ArrayList ret = new ArrayList();

        String base = url.getFile();

        if (base.startsWith("/")) {
            base = base.substring(1);
        }

        NamingEnumeration results = ctx.search(base, filter, constraints);

        while (results != null && results.hasMore()) {
            SearchResult sResult = (SearchResult) results.next();
            NamingEnumeration atrE = sResult.getAttributes().getAll();

            while (atrE.hasMoreElements()) {
                Attribute attr = (Attribute) atrE.next();
                Enumeration vals = attr.getAll();

                while (vals.hasMoreElements()) {
                    ret.add(vals.nextElement());
                }
            }
        }

        return ret;
    }
}
