package lbl.akenti.util;

import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.LineNumberReader;
import java.security.GeneralSecurityException;
import java.security.KeyFactory;
import java.security.Principal;
import java.security.PrivateKey;
import java.security.cert.Certificate;
import java.security.cert.X509Certificate;
import java.security.spec.PKCS8EncodedKeySpec;

/**
  * Helps to read PKCS8 unencrypted private keys
  * and X509 certs.
  *
  * @author Abdelilah Essiari
  */
public class P8Reader {
   private Certificate[] certs;
   private PrivateKey key;

   /**
     */
   public P8Reader() {
   }

   /**
     * Loads PKCS8 unencrypted  private key.
     *
     * @param fileName The path to the p8 file.
     */
   public void readPrivateKey(String fileName) 
     throws IOException, GeneralSecurityException {
     key = readPrivateKey(new FileInputStream(fileName));
   }

   /**
     * Loads PEM encoded x509 certs.
     *
     * @param fileName The path to the p8 file.
     */
   public void readX509Chain(String fileName) 
     throws IOException, GeneralSecurityException {
     certs = new X509CertReader().readX509Certs(fileName);

     if (certs.length == 1) {
       return;
     }

     // Find the subject of this chain.
     // This code assumes a chain.
     int count = 0;
     int j;
     
     for (j = 0; count != certs.length - 1 && j < certs.length; ++j) {
       X509Certificate temp = (X509Certificate) certs[j];
       Principal dn = temp.getSubjectDN();

       for (int i = 0; i < certs.length;++i) {
         if (i != j) {
           if (dn.equals(((X509Certificate) certs[i]).getIssuerDN())) {
             count = 0;
             break;
           } else {
             count++;
           }
         }
       }
     }
     
     if (count != certs.length - 1) {
       throw new GeneralSecurityException("BAD CHAIN");
     }
     
     Certificate temp = certs[0];
     certs[0] = certs[j - 1];
     certs[j - 1] = temp;
   }

   /**
     */
   public Principal getSubjectDN() {
     X509Certificate cert = (X509Certificate) getX509Certificate();

     return cert.getSubjectDN();
   }

   /**
     */
   public Principal getIssuerDN() {
     X509Certificate cert = (X509Certificate) getX509Certificate();

     return cert.getIssuerDN();
   }

   /**
     * A method to return the users X509 certificates.
     */
   public Certificate getX509Certificate() {
     return certs[0];
   }

   /**
     * A method to return the users X509 certificates.
     */
   public Certificate[] getX509CertificateChain() {
     return certs;
   }

   /**
     * A method to return the users PrivateKey from the p12 file.
     */
   public PrivateKey getPrivateKey() {
     return key;
   }

   private PrivateKey readPrivateKey(InputStream in) 
      throws IOException, GeneralSecurityException {
      InputStreamReader isr = new InputStreamReader(in);
      LineNumberReader lnr = new LineNumberReader(isr);
      StringBuffer sb = new StringBuffer();
      boolean found = false;

      String line = lnr.readLine();

      while (!found && line != null) {
        if (line.equals("-----BEGIN PRIVATE KEY-----")) {
          line = lnr.readLine();

          while (line != null) {
            sb.append(line);
            sb.append('\n');

            if (line.equals("-----END PRIVATE KEY-----")) {
              found = true;
              break;
            }

            line = lnr.readLine();
          }
        } else {
          line = lnr.readLine();
        }
      }
     
      if (!found) {
        return null;
      }
      
      byte[] der = Base64.decode(sb.toString());
      PKCS8EncodedKeySpec spec = new PKCS8EncodedKeySpec(der);
        
      try {
        KeyFactory fac = KeyFactory.getInstance("RSA");

        return fac.generatePrivate(spec);
      } catch(GeneralSecurityException ex) {
        KeyFactory fac = KeyFactory.getInstance("DSA");

        return fac.generatePrivate(spec);
      }
   } 
}  
