package lbl.akenti.util;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.TimeZone;

/**
 * A UTC time (also known as GMT, or Greenwich Mean Time).
 */
public class UTCTime {

  /**
   * Format originally used by akenti.
   */  
  public static String DEFAULT_FORMAT = "yyMMddHHmmss'Z'";
  /**
   * Format which is easy to read.
   */  
  public static String FRIENDLY_FORMAT = "E MMM dd HH:mm:ss z yyyy";
  /**
   * Format used by XSD in XML schema (reasonably easy to read).
   */  
  public static String XS_FORMAT = "yyyy-MM-dd'T'HH:mm:ss'Z'";
  
  private Date date;

  /**
   * Creates a defualt UTCTime object representing the
   * current time.
   */
  public UTCTime() {
    date = new Date();
  }

  /**
   * Creates a UTCTime object representing <CODE>date</CODE>.
   * @param date the date we wish to represent
   */
  public UTCTime(Date date) {
    this.date = date;
  }

  /**
   * Creates a UTCTime object representing the point in time
   * <CODE>offset</CODE> milliseconds after the current time.
   * @param offset the time offset in milliseconds
   */
  public UTCTime(long offset) {
    date = new Date();
    date.setTime(date.getTime() + offset);
  }

  /**
   * Creates a UTCTime object representing the point in time
   * <CODE>offset</CODE> milliseconds after <CODE>utc</CODE>.
   * @param utc the base time
   * @param offset the offset in milliseconds
   */
  public UTCTime(UTCTime utc, long offset) {
    date = new Date();
    date.setTime(utc.getTime() + offset);
  }

  /**
   * Creates a UTCTime object representing <CODE>rep</CODE>. If
   * <CODE>rep</CODE> is greater than 13 characters long it will
   * try to parse it as a date in <CODE>XS_FORMAT</CODE>;
   * otherwise it will assume the date is in DEFAULT_FORMAT.
   * @param rep the date in a recognized string format
   * @throws ParseException if no date can be parsed from the input
   * @see #DEFAULT_FORMAT
   * @see #XS_FORMAT
   */
  public UTCTime(String rep) throws ParseException {
    String format;
    if (rep.length() > 13) {
      format = XS_FORMAT;
    } else {
      format = DEFAULT_FORMAT;
    }
    SimpleDateFormat sdf  = new SimpleDateFormat(format);

    sdf.setTimeZone(TimeZone.getTimeZone("GMT"));
    date = sdf.parse(rep);
  }

  /**
   * Creates a UTCTime object representing <CODE>rep</CODE>
   * assuming it is in <CODE>format</CODE> format.
   * @see #DEFAULT_FORMAT
   * @see #FRIENDLY_FORMAT
   * @see #XS_FORMAT
   * @param format the format of <CODE>rep</CODE>
   * @param rep the date
   * @throws ParseException if no date can be parsed from the input
   */
  public UTCTime(String format, String rep) throws ParseException {
    SimpleDateFormat sdf  = new SimpleDateFormat(format);

    sdf.setTimeZone(TimeZone.getTimeZone("GMT"));
    date = sdf.parse(rep);
  }

  /**
   * Compares this UTCTime object to another and returns
   * a number indicating which one occurs first.
   * @param utc a time to compare to this one
   * @return 1 if this object represents a time after
   * <CODE>utc</CODE>; a 0 if they represent the same time; and
   * 0 otherwise.
   */
  public int compareTo(UTCTime utc) {
    long temp = (getTime() - utc.getTime()) / 1000;
    
    if (temp < 0) {
      return -1;
    } else if (temp == 0) {
      return 0;
    } else {
      return 1;
    }
  }

  /**
   * Returns this time in milliseconds since January
   * 1 st , 1970, 00:00:00.
   * @return the time in milliseconds.
   */
  public long getTime() {
    return date.getTime();
  }

  /**
   * Converts to an easy to read string.
   * @return this date in string form
   * @see #FRIENDLY_FORMAT
   */
  public String toFriendlyString() {
    return toString(FRIENDLY_FORMAT);
  }

  /**
   * Converts to default string format.
   * @return this date in string form
   * @see #DEFAULT_FORMAT
   */
  public String toString() {
    return toString(DEFAULT_FORMAT);
  }

  /**
   * Converts this date to a string in <CODE>format</CODE> format.
   * @param format a UTCTime string format
   * @return this date in string form
   * @see #DEFAULT_FORMAT
   * @see #FRIENDLY_FORMAT
   * @see #XS_FORMAT
   */
  public String toString(String format) {
    SimpleDateFormat sdf  = new SimpleDateFormat(format);
    
    sdf.setTimeZone(TimeZone.getTimeZone("GMT"));
    return sdf.format(date);
  }
}
